#!/bin/env python2.7
'''
Command line interface for the Climate Intercomparison Suite (CIS)
'''
import sys
import logging

from jasmin_cis.exceptions import CISError, NoDataInSubsetError

logger = logging.getLogger(__name__)

__author__ = "David Michel, Daniel Wallis and Duncan Watson-Parris"
__version__ = "V0R6M0"
__status__ = "Development"
__website__ = "http://proj.badc.rl.ac.uk/cedaservices/wiki/JASMIN/CommunityIntercomparisonSuite"

def __error_occurred(e):
    '''
    Wrapper method used to print error messages and exit the program.
    
    @param e: An error object or any string
    '''
    sys.stderr.write(str(e) + "\n")
    exit(1)

def __check_variable_is_valid(main_arguments, data, axis):
    '''
    Used for creating or appending to a dictionary of the format { variable_name : axis } which will later be used to assign
    the variable to the specified axis
    @param main_arguments: The arguments received from the parser
    @param data: A list of packed data objects
    @param var_axis_dict: A dictionary where the key will be the name of a variable and the value will be the axis it will be plotted on.
    @param axis: The axis on which to plot the variable on
    '''
    from jasmin_cis.exceptions import InvalidVariableError

    user_specified_variable = main_arguments.pop(axis + "axis")

    for data_item in data:
        if len(data_item.coords(name=user_specified_variable)) == 0 and len(data_item.coords(standard_name=user_specified_variable)) == 0 and data_item.standard_name != user_specified_variable and data_item.long_name != user_specified_variable:
            raise InvalidVariableError(user_specified_variable + " is not a valid variable")

    return user_specified_variable

def plot_cmd(main_arguments):
    '''
    Main routine for handling calls to the 'plot' command. 
    Reads in the data files specified and passes the rest of the arguments to the plot function.

    @param main_arguments:    The command line arguments
    '''
    from plotting.plot import Plotter
    from jasmin_cis.data_io.read import read_data
    import jasmin_cis.exceptions as ex
    from iris.exceptions import IrisError

    data = []
    try:
        # create a list of data object (ungridded or gridded(in that case, a Iris cube)), concatenating data from various files
        data = [ read_data(datagroup['filenames'], datagroup['variable'], datagroup['product']) for datagroup in main_arguments.datagroups ]
    except (IrisError, ex.InvalidVariableError, ex.ClassNotFoundError) as e:
        __error_occurred(e)
    except IOError as e:
        __error_occurred("There was an error reading one of the files: \n" + str(e))
    except MemoryError as e:
        __error_occurred("Not enough memory to read the data for the requested plot. Please either reduce the amount "
                         "of data to be plotted, increase the swap space available on your machine or use a machine "
                         "with more memory (for example the JASMIN facility).")

    main_arguments = vars(main_arguments)
    main_arguments.pop('command') # Remove the command argument now it is not needed
    plot_type = main_arguments.pop("type")
    output = main_arguments.pop("output")

    main_arguments["x_variable"] = __check_variable_is_valid(main_arguments, data, "x")
    main_arguments["y_variable"] = __check_variable_is_valid(main_arguments, data, "y")

    try:
        Plotter(data, plot_type, output, **main_arguments)
    except (ex.CISError, ValueError) as e:
        __error_occurred(e)
    except MemoryError:
        __error_occurred("Not enough memory to plot the data after reading it in. Please either reduce the amount "
                         "of data to be plotted, increase the swap space available on your machine or use a machine "
                         "with more memory (for example the JASMIN facility).")

def info_cmd(main_arguments):
    '''
    Main routine for handling calls to the 'info' command.
    Reads in the variables from the data file specified and lists them to stdout if no
    particular variable was specified, otherwise prints detailed information about each
    variable specified

    @param main_arguments:    The command line arguments (minus the info command)
    '''
    variables = main_arguments.variables
    filename = main_arguments.filename
    data_type = main_arguments.type

    from jasmin_cis.info import  info

    try:
        info(filename, variables, data_type)
    except CISError as e:
        __error_occurred(e)


def col_cmd(main_arguments):
    '''
    Main routine for handling calls to the co-locate ('col') command.

    @param main_arguments:    The command line arguments (minus the col command)
    '''
    from jasmin_cis.exceptions import ClassNotFoundError, CISError
    from jasmin_cis.col import Colocate
    from jasmin_cis.utils import add_file_prefix

    # Add a prefix to the output file so that we have a signature to use when we read it in again
    output_file = add_file_prefix("cis-", main_arguments.output + ".nc")

    try:
        col = Colocate(main_arguments.samplefiles, main_arguments.samplevariable, main_arguments.sampleproduct, output_file)
    except IOError as e:
        __error_occurred("There was an error reading one of the files: \n" + str(e))

    for input_group in main_arguments.datagroups:
        variable = input_group['variable']
        filenames = input_group['filenames']
        col_name = input_group['colocator'][0] if  input_group['colocator'] is not None else None
        col_options = input_group['colocator'][1] if  input_group['colocator'] is not None else None
        con_name = input_group['constraint'][0] if  input_group['constraint'] is not None else None
        con_options = input_group['constraint'][1] if  input_group['constraint'] is not None else None
        kern_name = input_group['kernel'][0] if  input_group['kernel'] is not None else None
        kern_options = input_group['kernel'][1] if  input_group['kernel'] is not None else None
        product = input_group["product"] if input_group["product"] is not None else None

        try:
            col.colocate(variable, filenames, col_name, col_options, con_name, con_options, kern_name, kern_options, product)
        except ClassNotFoundError as e:
            __error_occurred(str(e) + "\nInvalid co-location option.")
        except (CISError, IOError) as e:
            __error_occurred(e)


def subset_cmd(main_arguments):
    '''
    Main routine for handling calls to the subset command.

    @param main_arguments:    The command line arguments (minus the subset command)
    '''
    from jasmin_cis.subsetting.subset import Subset
    from jasmin_cis.utils import add_file_prefix

    if len(main_arguments.datagroups) > 1:
        __error_occurred("Subsetting can only be performed on one data group")
    input_group = main_arguments.datagroups[0]

    variable = input_group['variable']
    filenames = input_group['filenames']
    product = input_group["product"] if input_group["product"] is not None else None

    # Add a prefix to the output file so that we have a signature to use when we read it in again
    output_file = add_file_prefix("cis-", main_arguments.output + ".nc")
    subset = Subset(main_arguments.limits, output_file)
    try:
        subset.subset(variable, filenames, product)
    except (NoDataInSubsetError, CISError) as exc:
         __error_occurred(exc)


def version_cmd(_main_arguments):
    print "Using CIS version:", __version__, "("+__status__+")"


commands = {'plot': plot_cmd,
            'info': info_cmd,
            'col': col_cmd,
            'subset': subset_cmd,
            'version': version_cmd}


def main():
    '''
    The main method for the program.
    Sets up logging, parses the command line arguments and then calls the appropriate command with its arguments
    '''
    import os
    from parse import parse_args
    import logging, logging.config
    from datetime import datetime

    # configure logging
    logging.config.fileConfig( os.path.join(os.path.dirname(__file__), "logging.conf"))
    logging.captureWarnings(True) # to catch warning from 3rd party libraries

    # parse command line arguments
    arguments = parse_args()
    command = arguments.command

    logging.debug("CIS started at: " + datetime.now().strftime("%Y-%m-%d %H:%M"))
    logging.debug("Running command: " + command)
    logging.debug("With the following arguments: " + str(arguments))

    # execute command
    commands[command](arguments)


if __name__ ==  '__main__':
    main()



"""
Module to do integration tests of plots to files. Does not check the graph created is correct, only that a graph is
created without errors.
"""

from nose.tools import istest
from jasmin_cis.cis import plot_cmd
from jasmin_cis.parse import parse_args
from jasmin_cis.test.test_files.data import *


@istest
def should_do_scatter_plot_of_file_valid_aerosol_cci_file():
    # Actual file name: 20080612093821-ESACCI-L2P_AEROSOL-ALL-AATSR_ENVISAT-ORAC_32855-fv02.02.nc
    arguments = ['plot', valid_aerosol_cci_variable+':'+valid_aerosol_cci_filename, '--output',
                 valid_aerosol_cci_filename+'.png']
    main_arguments = parse_args(arguments)
    plot_cmd(main_arguments)

    # Remove plotted file, will throw an OSError if file was not created
    os.remove(valid_aerosol_cci_filename+'.png')

@istest
def should_do_contour_plot_of_valid_aerosol_cci_file():
    # Actual file name: 20080612093821-ESACCI-L2P_AEROSOL-ALL-AATSR_ENVISAT-ORAC_32855-fv02.02.nc
    # Trac issue #98
    arguments = ['plot', valid_aerosol_cci_variable+':'+valid_aerosol_cci_filename, '--output',
                 valid_aerosol_cci_filename+'.png', '--type', 'contour']
    main_arguments = parse_args(arguments)
    plot_cmd(main_arguments)

    # Remove plotted file, will throw an OSError if file was not created
    os.remove(valid_aerosol_cci_filename+'.png')

@istest
def should_do_contourf_plot_of_valid_aerosol_cci_file():
    # Actual file name: 20080612093821-ESACCI-L2P_AEROSOL-ALL-AATSR_ENVISAT-ORAC_32855-fv02.02.nc
    # Trac issue #98
    arguments = ['plot', valid_aerosol_cci_variable+':'+valid_aerosol_cci_filename, '--output',
                 valid_aerosol_cci_filename+'.png', '--type', 'contourf']
    main_arguments = parse_args(arguments)
    plot_cmd(main_arguments)

    # Remove plotted file, will throw an OSError if file was not created
    os.remove(valid_aerosol_cci_filename+'.png')


@istest
def should_do_scatter_plot_of_valid_2d_file():
    # Actual file name: xglnwa.pm.k8dec-k9nov.col.tm.nc
    arguments = ['plot', valid_1d_vairable+':'+valid_2d_filename+':product=Xglnwa', '--output', valid_2d_filename+'.png']
    main_arguments = parse_args(arguments)
    plot_cmd(main_arguments)

    # Remove plotted file, will throw an OSError if file was not created
    #os.remove(valid_2d_filename+'.png')


@istest
def should_do_scatter_plot_of_valid_2d_file_with_x_as_lat_y_as_long():
    # Actual file name: xglnwa.pm.k8dec-k9nov.col.tm.nc
    # Trac issue #153
    arguments = ['plot', valid_1d_vairable+':'+valid_2d_filename+':product=Xglnwa', '--xaxis', 'latitude', '--yaxis',
                 'longitude', '--output', valid_2d_filename+'.png']
    main_arguments = parse_args(arguments)
    plot_cmd(main_arguments)

    # Remove plotted file, will throw an OSError if file was not created
    os.remove(valid_2d_filename+'.png')


@istest
def should_do_heatmap_plot_of_valid_1d_file():
    # Actual file name: xglnwa.pm.k8dec-k9nov.vprof.tm.nc
    # Trac issue #145
    arguments = ['plot', valid_1d_vairable+':'+valid_1d_filename+':product=Xglnwa_vprof', '--output',
                 valid_1d_filename+'.png']
    main_arguments = parse_args(arguments)
    plot_cmd(main_arguments)

    # Remove plotted file, will throw an OSError if file was not created
    os.remove(valid_1d_filename+'.png')


@istest
def should_do_line_plot_of_valid_zonal_time_mean_cmip5_file():
    # Actual file name: rsutcs_Amon_HadGEM2-A_sstClim_r1i1p1_185912-188911.CMIP5.tm.zm.nc
    # Trac issue #130
    arguments = ['plot',
            valid_zonal_time_mean_CMIP5_variable+':'+valid_zonal_time_mean_CMIP5_filename+':product=NetCDF_CF_Gridded',
                 '--output', valid_zonal_time_mean_CMIP5_filename+'.png']
    main_arguments = parse_args(arguments)
    plot_cmd(main_arguments)

    # Remove plotted file, will throw an OSError if file was not created
    os.remove(valid_zonal_time_mean_CMIP5_filename+'.png')



from jasmin_cis.plotting.generic_plot import Generic_Plot

class Heatmap(Generic_Plot):
    def plot(self):
        '''
        Plots a heatmap
        '''
        from jasmin_cis.exceptions import InvalidNumberOfDatagroupsSpecifiedError
        if len(self.packed_data_items) != 1:
            raise InvalidNumberOfDatagroupsSpecifiedError("Invalid number of datagroups specified. Only one datagroup can be plotted for a heatmap.")

        if self.is_map(): self.mplkwargs["latlon"] = True

        self.plotting_library.pcolormesh(self.unpacked_data_items[0]["x"], self.unpacked_data_items[0]["y"], self.unpacked_data_items[0]["data"], *self.mplargs, **self.mplkwargs)

        self.mplkwargs.pop("latlon", None)

    def set_default_axis_label(self, axis):
        return self.set_3daxis_label(axis)

    def create_legend(self):
        pass

    def format_plot(self):
        self.format_time_axis()
        self.format_3d_plot()


'''
Module to test the col command of the parser
The parse raises a SystemExit exception with code 2 if it fails to parse.
Some tests therefore ignore SystemExit exceptions with code 2 if they are expected.
'''
from nose.tools import istest, eq_
from jasmin_cis.parse import parse_args
from jasmin_cis.test.test_files.data import *

@istest
def can_specify_one_valid_samplefile_and_one_complete_datagroup():
    args = ["col", valid_1d_filename, "variable:"+valid_1d_filename+":col:con:nn"]
    args = parse_args(args)
    eq_(valid_1d_filename, args.samplegroup)
    eq_([{"filenames" : [valid_1d_filename], "variable" : "variable", "colocator" : ("col",{}), "constraint" : ("con",{}), "kernel" : ("nn",{}), "product" : None}],
        args.datagroups)

@istest
def can_specify_one_valid_samplefile_and_one_datafile_without_other_options():
    args = ["col", valid_1d_filename, "variable:"+valid_1d_filename]
    args = parse_args(args)
    eq_(valid_1d_filename, args.samplegroup)
    eq_([{"filenames" : [valid_1d_filename], "variable" : "variable", "colocator" : None, "constraint" : None, "kernel" : None, "product" : None}],
        args.datagroups)

@istest
def can_specify_one_valid_samplefile_and_many_datagroups():
    args = ["col", valid_1d_filename, 
            "variable:"+valid_1d_filename,
            "variable:"+valid_1d_filename+':col',
            "variable:"+valid_1d_filename+'::con',
            "variable:"+valid_1d_filename+':::nn',
            "variable:"+valid_1d_filename+':col::nn']
    args = parse_args(args)
    eq_(valid_1d_filename, args.samplegroup)
    eq_([{"filenames" : [valid_1d_filename], "variable" : "variable", "colocator" : None, "constraint" : None, "kernel" : None, "product" : None},
         {"filenames" : [valid_1d_filename], "variable" : "variable", "colocator" : ('col',{}), "constraint" : None, "kernel" : None, "product" : None},
         {"filenames" : [valid_1d_filename], "variable" : "variable", "colocator" : None, "constraint" : ('con',{}), "kernel" : None, "product" : None},
         {"filenames" : [valid_1d_filename], "variable" : "variable", "colocator" : None, "constraint" : None, "kernel" : ('nn',{}), "product" : None},
         {"filenames" : [valid_1d_filename], "variable" : "variable", "colocator" : ('col',{}), "constraint" : None, "kernel" : ('nn',{}), "product" : None}],
        args.datagroups)


@istest
def can_specify_one_valid_samplefile_and_one_datafile_with_internal_options():
    args = ["col", valid_1d_filename, "variable:"+valid_1d_filename+"::SepConstraint,h_sep=1500,v_sep=22000,t_sep=5000:nn"]
    args = parse_args(args)
    eq_(valid_1d_filename, args.samplegroup)
    eq_([{"filenames" : [valid_1d_filename], "variable" : "variable", "colocator" : None,
          "constraint" : ('SepConstraint',{'h_sep':'1500','v_sep':'22000','t_sep':'5000'}), "kernel" : ('nn',{}), "product" : None}], args.datagroups)


@istest
def should_raise_error_with_missing_arguments():
    try:
        args = ["col", valid_1d_filename]
        args = parse_args(args)
        assert False
    except SystemExit as e:
        if e.code != 2:
            raise e


from collections import namedtuple


class SubsetLimits(namedtuple('SubsetLimits', ['start', 'end', 'is_time'])):
    """Holds the start and end values for subsetting limits.
    is_date indicates whether the limits are date/times - None if unknown
    @ivar start: subsetting limit start
    @type start: str
    @ivar end: subsetting limit end
    @type end: str
    @ivar is_time: indicates whether the limits apply to a time dimension: None if not known
    @type is_type: bool
    """
    pass



import matplotlib.pyplot as plt
from numpy import arange
from numpy.random import random
from math import ceil, floor

w = 2

#d1 = arange(start=-5, stop=20.6, step=0.5)
d1 = random(30)*50 - 5
print d1

#d2 = arange(start=0,stop=50, step=1)
d2 = random(10)*20
print d2

# GETTING THE TOTAL RANGE
min_v = min(d1.min(), d2.min())
max_v = max(d1.max(), d2.max())
r = ceil(max_v - min_v)
print r

w = None
if w is None:
    w = ceil((max_v-min_v)/10)

# CREATING AN ARRAY OF BIN EDGES
bins = arange(start=min_v,stop=min_v+r+w,step=w)
print bins

plt.hist(d1, bins=bins, histtype="step")
plt.hist(d2, bins=bins, histtype="step")
plt.show()



from jasmin_cis.data_io.netcdf import *

if __name__ == '__main__':
# get a list of files to read
    #folder = "/home/david/Data"
    #filenames = glob(folder + "/*" + "CS_2B-CWC-RVOD_GRANULE_P_R04_E02.hdf")
    # WALDMfilenames = "/home/duncan/xenida.pah94?0.nc"
    # WALDM filename = '/home/duncan/xenida.pah9430.nc'
    filenames = "/home/daniel/CCI_Files/*"
    filename = "/home/daniel/CCI_Files/20080915012939-ESACCI-L2P_AEROSOL-ALL-AATSR_ENVISAT-ORAC_34210-fv02.02.nc"
    # WALDM var, coords = read_many_files(filenames, 'tendency_of_mass_fraction_of_cloud_liquid_water_in_air', 'time_1')
    aod550, aodcoords = read_many_files(filenames, 'AOD550', "pixel_number")
#    lat, latcoords = read_many_files(filenames, 'lat', "pixel_number")
 #   lon, loncoords = read_many_files(filenames, 'lon', "pixel_number")

    # read the data
    #data = readin_cloudsat_cwc_rvod(filenames,['RVOD_liq_water_content','Height','RVOD_ice_water_content'])

    # print results
#    print "\nkeys are: " , data.keys()
#
#    for key, value in data.iteritems():
#        try:
#            print "\n" + key + " :", get_metadata(value)['attributes'], ", dims: ", get_metadata(value)['dimensions']
#        except KeyError:
#            pass

    #var = data['tendency_of_mass_fraction_of_cloud_liquid_water_in_air']

    #coords = [ read_many_files(filenames, dim) for dim in var.dimensions ]

    print "aod550"
    print aod550
  #  print "lat"
   # print lat
    #print "lon"
    #print lon

    print "aodcoords"
    for coord in aodcoords:
        print coord
    #print "latcoords"
    #for coord in latcoords:
     #   print coord
    #print "loncoords"
    #for coord in loncoords:
     #   print coord

    print "aoddata"
    aoddata = get_data(aod550)
    print aoddata
    #print "latdata"
    #latdata = get_data(lat)
    #print latdata
    #print "londata"
    #londata = get_data(lon)
    #print londata
    #for item in data:
    #    if item != "--":
    #        print item



"""
Reading in data from MODIS L3
"""
import numpy as np
from pyhdf.error import HDF4Error
from pyhdf.SD import *
from jasmin_cis.test.harness import hdf
import re

def readin_MODIS_L3(filenames,sds,outdata=None):
    '''Reads in MODIS L3 data from the collection 5 daily files on cloud'''
    if outdata == None:
        outdata = {}

    names = sds + ['XDim','YDim']

    try:
        l3dict = hdf.read_hdf4(filenames[0],names)
        for name in names:
            try:
                outdata[name] = np.hstack((outdata[name],l3dict[name]))
            except KeyError:
                #print KeyError, ' in readin_MODIS_L3', name
                outdata[name] = l3dict[name]
    except HDF4Error:
        print HDF4Error, filenames
    return outdata


if __name__ ==  '__main__':

    import matplotlib.pyplot as plt
    from mpl_toolkits.basemap import Basemap
    from glob import glob

    # get a list of files to read
    folder = "/home/david/Data"
    filenames = glob(folder + "/*" + "MYD08_D3" + "*.hdf")
    filenames = np.sort(filenames)

    #Get some level 3 data (daily mean)
    L3_data = readin_MODIS_L3(filenames,['Cloud_Water_Path_Combined_Mean'])
    #The L3 data does have some lat/lon data in the file, but as it is regularly gridded, I
    # usually just plot it using imshow.
    plt_l3 = L3_data['Cloud_Water_Path_Combined_Mean'].squeeze()[::-1]

    # do plotting
    m = Basemap(-180,-90,180,90)
    m.imshow(plt_l3,vmax=600,interpolation='nearest')
    m.drawcoastlines()
    plt.title('MYD08_D3 - Cloud Water Path Combined')
    plt.xticks([-180,-90,0,90,180],[-180,-90,0,90,180])
    plt.yticks([-60,-30,0,30,60],[-60,-30,0,30,60,90])
    plt.xlabel('Longitude')
    plt.ylabel('Latitude')
    cb = plt.colorbar(shrink=0.8)
    cb.set_label('gm $^{-2}$')

    plt.show()







from jasmin_cis.plotting.generic_plot import Generic_Plot

class Comparative_Scatter(Generic_Plot):
    def plot(self):
        from jasmin_cis.exceptions import InvalidNumberOfDatagroupsSpecifiedError
        if len(self.packed_data_items) == 2:
            # Add y=x line
            ax = self.matplotlib.gca()
            if self.plot_args.get("logx", False) and self.plot_args.get("logy", False):
                import numpy.ma as ma
                positive_item0 = ma.array(self.unpacked_data_items[0]["data"], mask=self.unpacked_data_items[0]["data"]<=0)
                positive_item1 = ma.array(self.unpacked_data_items[1]["data"], mask=self.unpacked_data_items[1]["data"]<=0)
                min_val = min(positive_item0.min(), positive_item1.min())
                max_val = max(positive_item0.max(), positive_item1.max())
            else:
                min_val = min(self.unpacked_data_items[0]["data"].min(), self.unpacked_data_items[1]["data"].min())
                max_val = max(self.unpacked_data_items[0]["data"].max(), self.unpacked_data_items[1]["data"].max())
            y_equals_x_array = [min_val, max_val]
            ax.plot(y_equals_x_array, y_equals_x_array, color = "black", linestyle = "dashed")

            scatter_size = self.plot_args.get("itemwidth", 1) if self.plot_args.get("itemwidth", 1) is not None else 1
            datagroup = self.plot_args["datagroups"][0]
            if datagroup["itemstyle"]:
                self.mplkwargs["marker"] = datagroup["itemstyle"]
            else:
                self.mplkwargs.pop("marker", None)

            if datagroup["color"]:
                self.mplkwargs["color"] = datagroup["color"]
            else:
                self.mplkwargs.pop("color", None)

            ax.scatter(self.unpacked_data_items[0]["data"], self.unpacked_data_items[1]["data"], s=scatter_size, edgecolors = "none", *self.mplargs, **self.mplkwargs)
        else:
            raise InvalidNumberOfDatagroupsSpecifiedError("Comparative scatter requires two datagroups")

    def unpack_data_items(self):
        return self.unpack_comparative_data()

    def calculate_axis_limits(self, axis, min_val, max_val, step):
        if axis == "x":
            axis_index = 0
        elif axis == "y":
            axis_index = 1

        calculated_min, calculated_max = self.calculate_min_and_max_values_of_array_including_case_of_log(axis, self.unpacked_data_items[axis_index]["data"])

        valrange = {}
        valrange[axis + "min"] = calculated_min if min_val is None else min_val
        valrange[axis + "max"] = calculated_max if max_val is None else max_val
        valrange[axis + "step"] = step

        return valrange

    def format_plot(self):
        # We don't format the time axis here as we're only plotting data against data
        self.format_2d_plot()

    def set_default_axis_label(self, axis):
        self.set_default_axis_label_for_comparative_plot(axis)

    def create_legend(self):
        pass

    def is_map(self):
        return False



from jasmin_cis.plotting.generic_plot import Generic_Plot

class Contour_Plot(Generic_Plot):
    #'contour' : PlotType(1, 2, plot_contour),
    def plot(self):
        '''
        Plots a contour plot
        Stores the plot in a list to be used for when adding the legend

        @param data_item:    A dictionary containing the x coords, y coords and data as arrays
        '''
        self.contour_plot(False)

    def set_default_axis_label(self, axis):
        return self.set_3daxis_label(axis)

    def format_plot(self):
        self.format_time_axis()
        self.format_3d_plot()



'''
Module to test the parser
The parse raises a SystemExit exception with code 2 if it fails to parse.
Each test therefore ignores SystemExit exceptions with code 2 as they are expected.
'''
from nose.tools import istest, eq_
from jasmin_cis.parse import parse_args, expand_file_list
from test_files.data import *
from jasmin_cis.plotting.plot import Plotter
import argparse

@istest
def order_is_preserved_when_specifying_individual_files():
    parser = argparse.ArgumentParser()
    files = expand_file_list(valid_1d_filename + "," + valid_2d_filename, parser)
    eq_(files, [valid_1d_filename, valid_2d_filename])
    files = expand_file_list(valid_2d_filename + "," + valid_1d_filename, parser)
    eq_(files, [valid_2d_filename, valid_1d_filename])

@istest
def directories_are_sorted():
    parser = argparse.ArgumentParser()
    files = expand_file_list(test_directory, parser)
    eq_(files, test_directory_files)

@istest
def wildcarded_files_are_sorted():
    parser = argparse.ArgumentParser()
    files = expand_file_list(os.path.join(test_directory,'test_file_for_parser_*'), parser)
    eq_(files, test_directory_files)

@istest
def order_is_preserved_when_specifying_files_even_when_wildcards_and_directories_are_specified_too():
    parser = argparse.ArgumentParser()
    files = expand_file_list(valid_1d_filename + "," + valid_2d_filename + "," + os.path.join(test_directory,'test_file_for_parser_1') + "," + valid_cloud_cci_filename + "," + test_directory, parser)
    eq_(files, [valid_1d_filename, valid_2d_filename, test_directory_file1, valid_cloud_cci_filename, test_directory_file2, test_directory_file3])

@istest
def can_specify_one_valid_filename_and_a_directory():
    parser = argparse.ArgumentParser()
    files = expand_file_list(valid_1d_filename+','+test_directory, parser)
    eq_(files, [valid_1d_filename] + test_directory_files)

@istest
def can_specify_a_directory():
    parser = argparse.ArgumentParser()
    files = expand_file_list(test_directory, parser)
    eq_(files, test_directory_files)

@istest
def can_specify_a_file_with_wildcards():
    parser = argparse.ArgumentParser()
    files = expand_file_list(os.path.join(test_directory,'test_file_for_parser_*'), parser)
    eq_(files,test_directory_files)
    files = expand_file_list(os.path.join(test_directory,'*_1'), parser)
    eq_(files,[test_directory_file1])
    files = expand_file_list(os.path.join(test_directory,'test_file_for_parser_?'), parser)
    eq_(files,test_directory_files)
    files = expand_file_list(os.path.join(test_directory,'test_file_for_parser_[0-9]'), parser)
    eq_(files,test_directory_files)

@istest
def can_specify_one_valid_filename_and_a_wildcarded_file():
    parser = argparse.ArgumentParser()
    files = expand_file_list(valid_1d_filename+','+os.path.join(test_directory,'test_file_for_parser_[0-9]'), parser)
    eq_(files,[valid_1d_filename] + test_directory_files)

@istest
def duplicate_files_are_not_returned_from_expand_file_list():
    parser = argparse.ArgumentParser()
    files = expand_file_list(os.path.join(test_directory,'test_file_for_parser_1')+','+os.path.join(test_directory,'test_file_for_parser_[0-9]'), parser)
    eq_(files,test_directory_files)

@istest
def can_specify_one_valid_filename():
    args = ["plot", valid_variable_in_valid_filename+":"+valid_1d_filename]
    parse_args(args)
    
@istest
def should_raise_error_with_one_invalid_filename():
    try:
        args = ["plot", valid_variable_in_valid_filename+":"+invalid_filename]
        parse_args(args)
        assert False
    except SystemExit as e:
        if e.code != 2:
            raise e
    
@istest
def can_specify_more_than_one_valid_filename():
    args = ["plot", valid_variable_in_valid_filename+":"+valid_1d_filename+','+netcdf_file_with_incorrect_file_extension]
    parse_args(args)
    
@istest
def should_raise_error_with_a_mixture_of_valid_and_invalid_filenames():
    try:
        args = ["plot", valid_variable_in_valid_filename+":"+valid_1d_filename+','+invalid_filename]
        parse_args(args)
        assert False
    except SystemExit as e:
        if e.code != 2:
            raise e        
    
@istest
def can_specify_valid_chart_type():
    args = ["plot", valid_variable_in_valid_filename+":"+valid_1d_filename, "--type", Plotter.plot_types.keys()[0]]
    parse_args(args)

@istest
def should_raise_error_with_an_invalid_chart_type():
    try:
        args = ["plot", valid_variable_in_valid_filename+":"+valid_1d_filename, "--type", "dfgdfgdfgdfgdfgdf"]
        parse_args(args)
        assert False
    except SystemExit as e:
        if e.code != 2:
            raise e    
    
@istest
def should_raise_error_with_more_than_one_chart_type():
    try:
        args = ["plot", valid_variable_in_valid_filename+":"+valid_1d_filename, "--type", Plotter.plot_types.keys()[0], Plotter.plot_types.keys()[1]]
        parse_args(args)
        assert False
    except SystemExit as e:
        if e.code != 2:
            raise e

@istest
def can_specify_more_than_one_variable():
    args = ["plot", valid_variable_in_valid_filename+":"+valid_1d_filename, valid_variable_in_valid_filename+":"+valid_1d_filename]
    parse_args(args)
    
@istest
def should_raise_error_when_no_variable_is_specified():
    try:
        args = ["plot", valid_1d_filename]
        parse_args(args)
        assert False
    except SystemExit as e:
        if e.code != 2:
            raise e    
        
@istest
def should_raise_error_with_invalid_line_width():
    try:   
        args = ["plot", valid_variable_in_valid_filename+":"+valid_1d_filename, "--itemwidth", "4a0"]
        parse_args(args) 
        assert False
    except SystemExit as e:
        if e.code != 2:
            raise e  
        
@istest
def should_raise_error_with_invalid_line_style():   
    try:   
        args = ["plot", valid_variable_in_valid_filename+":"+valid_1d_filename, "--linestyle", "4a0"]
        parse_args(args) 
        assert False
    except SystemExit as e:
        if e.code != 2:
            raise e 



'''
Module used for parsing
'''
import argparse
import re
import sys
import os.path
from plotting.plot import Plotter
import logging

def initialise_top_parser():
    '''
    The parser to which all arguments are initially passed
    '''
    parser = argparse.ArgumentParser("cis")
    subparsers = parser.add_subparsers(dest='command')
    plot_parser = subparsers.add_parser("plot", help = "Create plots")
    add_plot_parser_arguments(plot_parser)
    info_parser = subparsers.add_parser("info", help = "Get information about a file")
    add_info_parser_arguments(info_parser)
    col_parser = subparsers.add_parser("col", help = "Perform colocation")
    add_col_parser_arguments(col_parser)
    subset_parser = subparsers.add_parser("subset", help = "Perform subsetting")
    add_subset_parser_arguments(subset_parser)
    subparsers.add_parser("version", help = "Display the CIS version number")
    return parser


def add_plot_parser_arguments(parser):

    from jasmin_cis.data_io.products.AProduct import AProduct
    import jasmin_cis.plugin as plugin
    product_classes = plugin.find_plugin_classes(AProduct, 'jasmin_cis.data_io.products.products', verbose=False)

    parser.add_argument("datagroups", metavar = "Input datagroups", nargs = "+",
                        help = "The datagroups to be plotted, in the format: variable:filenames:colour:style:label:product, where the last four arguments are optional, colour is any valid html colour (e.g. red) and product is one of " + str([cls().__class__.__name__ for cls in product_classes]))
    parser.add_argument("-o", "--output", metavar = "Output filename", nargs = "?", help = "The filename of the output file for the plot image")
    parser.add_argument("--type", metavar = "Chart type", nargs = "?", help = "The chart type, one of: " + str(Plotter.plot_types.keys()))

    parser.add_argument("--xlabel", metavar = "X axis label", nargs = "?", help = "The label for the x axis")
    parser.add_argument("--ylabel", metavar = "Y axis label", nargs = "?", help = "The label for the y axis")
    parser.add_argument("--cbarlabel", metavar = "Colour bar label", nargs = "?", help = "The label for the colour bar")

    parser.add_argument("--xtickangle", metavar = "X tick angle", nargs = "?", help = "The angle (in degrees) of the ticks on the x axis")
    parser.add_argument("--ytickangle", metavar = "Y tick angle", nargs = "?", help = "The angle (in degrees) of the ticks on the y axis")

    parser.add_argument("--title", metavar = "Chart title", nargs = "?", help = "The title for the chart")    
    parser.add_argument("--itemwidth", metavar = "Item width", nargs = "?", help = "The width of an item. Unit are points in the case of a line, and point^2 in the case of a scatter point.")
    parser.add_argument("--fontsize", metavar = "Font size", nargs = "?", help = "The size of the font in points")
    parser.add_argument("--cmap", metavar = "Colour map", nargs = "?", help = "The colour map used, e.g. RdBu")
    parser.add_argument("--height", metavar = "Plot height", nargs = "?", help = "The height of the plot in inches")
    parser.add_argument("--width", metavar = "Plot width", nargs = "?", help = "The width of the plot in inches")

    parser.add_argument("--xmin", metavar = "Minimum x", nargs = "?", help = "The minimum x value to plot")
    parser.add_argument("--xmax", metavar = "Maximum x", nargs = "?", help = "The maximum x value to plot")
    parser.add_argument("--xstep", metavar = "X step", nargs = "?", help = "The step of the x axis")

    parser.add_argument("--ymin", metavar = "Minimum y", nargs = "?", help = "The minimum y value to plot")
    parser.add_argument("--ymax", metavar = "Maximum y", nargs = "?", help = "The maximum y value to plot")
    parser.add_argument("--ystep", metavar = "Y step", nargs = "?", help = "The step of the y axis")

    parser.add_argument("--vmin", metavar = "Minimum value", nargs = "?", help = "The minimum value to plot")
    parser.add_argument("--vmax", metavar = "Maximum value", nargs = "?", help = "The maximum value to plot")
    parser.add_argument("--vstep", metavar = "X value", nargs = "?", help = "The step of the colour bar")

    parser.add_argument("--xbinwidth", metavar = "Histogram x axis bin width", nargs = "?", help = "The width of the bins on the x axis of a histogram")
    parser.add_argument("--ybinwidth", metavar = "Histogram y axis bin width", nargs = "?", help = "The width of the bins on the y axis of a histogram")

    parser.add_argument("--cbarorient", metavar = "Colour bar orientation", default = "vertical", nargs = "?", help = "The orientation of the colour bar, either horizontal or vertical")
    parser.add_argument("--nocolourbar", metavar = "Hides the colour bar", default = "False", nargs = "?", help = "Does not show the colour bar")

    parser.add_argument("--logx", metavar = "Log (base 10) scale on X axis", default = "False", nargs = "?", help = "Uses a log scale (base 10) on the x axis")
    parser.add_argument("--logy", metavar = "Log (base 10) scale on Y axis", default = "False", nargs = "?", help = "Uses a log scale (base 10) on the y axis")
    parser.add_argument("--logv", metavar = "Log (base 10) scale for values", default = "False", nargs = "?", help = "Uses a log scale (base 10) on the colour bar")

    parser.add_argument("--grid", metavar = "Show grid", default = "False", nargs = "?", help = "Shows grid lines on the plot")
    parser.add_argument("--xaxis", metavar = "Variable on x axis", nargs="?", help="Name of variable to use on the x axis")
    parser.add_argument("--yaxis", metavar = "Variable on y axis", nargs="?", help="Name of variable to use on the y axis")

    parser.add_argument("--coastlinescolour", metavar = "Coastlines Colour", nargs = "?", help = "The colour of the coastlines on a map. Any valid html colour (e.g. red)")
    return parser


def add_info_parser_arguments(parser):
    parser.add_argument("filename", metavar = "Filename", help = "The filename of the file to inspect")
    parser.add_argument("-v", "--variables", metavar = "Variable(s)", nargs = "+", help = "The variable(s) to inspect")
    parser.add_argument("--type", metavar = "type of HDF data", nargs="?", help="Can be 'VD' or 'SD'. Use 'All' for both.")
    return parser


def add_col_parser_arguments(parser):
    parser.add_argument("samplegroup", metavar = "SampleGroup", help = "A string of the format filename:variable:product where the variable and product are optional")
    parser.add_argument("datagroups", metavar = "DataGroups", nargs = "+", help = "Variable to colocate with filenames and other options split by a colon")
    parser.add_argument("-o", "--output", metavar = "Output filename", default = "out", nargs = "?", help = "The filename of the output file for the plot image")
    return parser


def add_subset_parser_arguments(parser):
    parser.add_argument("datagroups", metavar="DataGroup", nargs=1,
                        help = "Variable to subset with filenames and optional product separated by colon(s)")
    parser.add_argument("subsetranges", metavar="SubsetRanges",
                        help = "Dimension ranges to use for subsetting")
    parser.add_argument("-o", "--output", metavar="Output filename", default="out", nargs="?",
                        help = "The filename of the output file")
    return parser


def expand_file_list(filenames, parser):
    '''

    @param filenames: A string which is a comma seperated list of filenames, wildcarded filenames or directories
    @param parser: A reference parser for raising errors on
    @return: A flat list of files which exist - with no duplicate
    '''
    from glob import glob
    from jasmin_cis.utils import OrderedSet
    if not filenames:
        parser.error("Please specify at least one filename")
    input_list = filenames.split(',')

    # Ensure we don't get duplicates by making file_set a set
    file_set = OrderedSet()
    for element in input_list:
        if any(wildcard in element for wildcard in ['*', '?',']','}']):
            filenames = glob(element)
            filenames.sort()
            for filename in filenames:
                file_set.add(filename)
        elif os.path.isdir(element):
            filenames = os.listdir(element)
            filenames.sort()
            for a_file in filenames:
                full_file = os.path.join(element, a_file)
                if os.path.isfile(full_file):
                    file_set.add(full_file)
        elif os.path.isfile(element):
            file_set.add(element)
        else:
            parser.error("'" + element + "' is not a valid filename")

    # Check we matched at least one file
    if not file_set:
        parser.error("No files found which match: " + str(filenames))

    # Cast set to a list to make it easier to index etc. later on
    alist = list(file_set)

    logging.info("Identified input file list: " + str(alist))

    return alist


def check_file_exists(filename, parser):
    if not os.path.isfile(filename):
        parser.error("'" + filename + "' is not a valid filename")


def parse_float(arg, name, parser):
    '''
    Tries to parse a string as a float.
    
    @param arg:    The arg to parse as a float
    @param name:   A description of the argument used for error messages
    @param parser: The parser used to report an error message
    @return The parsed float if succeeds or the original argument if fails
    '''
    if arg:
        try:
            arg = float(arg)
            return arg
        except ValueError:
            parser.error("'" + arg + "' is not a valid " + name)
            return None


def parse_int(arg, name, parser):
    '''
    Tries to parse a string as an integer.

    @param arg:    The arg to parse as an integer
    @param name:   A description of the argument used for error messages
    @param parser: The parser used to report an error message
    @return: The parsed integer if succeeds or None if fails
    '''
    if arg:
        try:
            arg = int(arg)
            return arg
        except ValueError:
            parser.error("'" + arg + "' is not a valid " + name)
            return None


def check_product(product, parser):
    from jasmin_cis.data_io.products.AProduct import AProduct
    import jasmin_cis.plugin as plugin
    if product:
        product_classes = plugin.find_plugin_classes(AProduct, 'jasmin_cis.data_io.products.products')
        product_names = [cls().__class__.__name__ for cls in product_classes]
        if product not in product_names:
            parser.error(product + " is not a valid product. Please use one of " + str(product_names))
    else:
        product = None
    return product

def get_plot_datagroups(datagroups, parser):
    '''
    @param datagroups:    A list of datagroups (possibly containing colons)
    @param parser:       The parser used to report errors    
    @return The parsed datagroups as a list of dictionaries
    '''
    from collections import namedtuple
    DatagroupOptions = namedtuple('DatagroupOptions',[ "variable", "filenames", "color", "edgecolor", "itemstyle", "label", "product"])
    datagroup_options = DatagroupOptions(check_is_not_empty, expand_file_list, check_color, check_color, check_nothing, check_nothing, check_product)
    return parse_colon_and_comma_separated_arguments(datagroups, parser, datagroup_options, compulsary_args=2)


def get_col_datagroups(datagroups, parser):
    '''
    @param datagroups:    A list of datagroups (possibly containing colons)
    @param parser:       The parser used to report errors
    @return The parsed datagroups as a list of dictionaries
    '''
    from collections import namedtuple
    DatagroupOptions = namedtuple('DatagroupOptions',["variable", "filenames", "colocator", "constraint", "kernel", "product"])
    datagroup_options = DatagroupOptions(check_is_not_empty, expand_file_list, extract_method_and_args, extract_method_and_args, extract_method_and_args, check_product)

    return parse_colonic_arguments(datagroups, parser, datagroup_options, min_args=2)


def get_col_samplegroup(samplegroup, parser):
    '''
    @param samplegroups:    A list of datagroups (possibly containing colons)
    @param parser:       The parser used to report errors
    @return The parsed samplegroups as a list of dictionaries
    '''
    from collections import namedtuple
    DatagroupOptions = namedtuple('SamplegroupOptions',[ "filenames", "variable", "product"])
    samplegroup_options = DatagroupOptions(expand_file_list, check_nothing, check_product)

    return parse_colonic_arguments(samplegroup, parser, samplegroup_options)


def get_subset_datagroups(datagroups, parser):
    '''
    @param datagroups:    A list of datagroups (possibly containing colons)
    @param parser:       The parser used to report errors
    @return The parsed datagroups as a list of dictionaries
    '''
    from collections import namedtuple
    DatagroupOptions = namedtuple('DatagroupOptions', ["variable", "filenames", "product"])
    datagroup_options = DatagroupOptions(check_is_not_empty, expand_file_list, check_product)

    return parse_colonic_arguments(datagroups, parser, datagroup_options, min_args=2)


def get_subset_limits(subsetlimits, parser):
    '''
    @param subsetlimits:  List of subset limit strings
    @param parser:        The parser used to report errors
    @return The parsed datagroups as a list of dictionaries
    '''
    from jasmin_cis.parse_datetime import parse_datetime, parse_as_number_or_datetime
    from jasmin_cis.subsetting.subset_limits import SubsetLimits

    # Split into the limits for each dimension.
    split_input = split_outside_brackets(subsetlimits)
    if len(split_input) == 0:
        parser.error("Limits for at least one dimension must be specified for subsetting")

    limit_dict = {}
    for seg in split_input:
        # Parse out dimension name and limit value strings; the expected format is:
        # <dim_name>=[<start_value>,<end_value>]
        # or
        # <dim_name>=[<start_value>]
        match = re.match(r'(?P<dim>[^=]+)=\[(?P<start>[^],]+)(?:,(?P<end>[^],]+))?\]$', seg)
        if match is None or match.group('dim') is None or match.group('start') is None:
            parser.error(
                "A dimension for subsetting does not have dimension name, start value and/or end value specified")
        else:
            dim_name = match.group('dim')
            limit1 = match.group('start')
            if match.group('end') is None:
                limit2 = limit1
            else:
                limit2 = match.group('end')

            # If the dimension is specified as x, y, z, or t, assume that the dimension is spatial or temporal in the
            # obvious way. Otherwise, parse what is found as a date/time or number.
            is_time = None
            if dim_name.lower() == 't':
                limit1_parsed = parse_datetime(limit1, 'subset range start date/time', parser)
                limit2_parsed = parse_datetime(limit2, 'subset range end date/time', parser)
                is_time = True
            elif dim_name.lower() in ['x', 'y', 'z']:
                limit1_parsed = parse_float(limit1, 'subset range start coordinate', parser)
                limit2_parsed = parse_float(limit2, 'subset range start coordinate', parser)
                is_time = False
            else:
                limit1_parsed = parse_as_number_or_datetime(limit1, 'subset range start coordinate', parser)
                limit2_parsed = parse_as_number_or_datetime(limit2, 'subset range start coordinate', parser)
            limit_dict[dim_name] = SubsetLimits(limit1_parsed, limit2_parsed, is_time)
    return limit_dict


def parse_colonic_arguments(inputs, parser, options, min_args=1):
    '''
    @param inputs:    A list of strings, each in the format a:b:c:......:n where a,b,c,...,n are arguments
    @param parser:    The parser used to raise an error if one occurs
    @param options:   The possible options that each input can take. If no value is assigned to a particular option, then it is assigned None
    @param min_args:   The minimum number of arguments to expect - we can't say which arguments are compulsory, just how many are
    @return A list of dictionaries containing the parsed arguments
    '''
    input_dicts = []
    
    for input_string in inputs:
        split_input = input_string.split(":")
        if len(split_input) < min_args:
            parser.error("A mandatory data group option is missing")
        input_dict = {}
        
        for i, option in enumerate(options._asdict().keys()):
            try:
                current_option = split_input[i]
                input_dict[option] = options[i](current_option, parser)
            except IndexError:
                input_dict[option] = None
        
        input_dicts.append(input_dict)
    return input_dicts


def parse_colon_and_comma_separated_arguments(inputs, parser, options, compulsary_args):
    '''
    @param inputs:    A list of strings, each in the format a:b:c:......:n where a,b,c,...,n are arguments
    @param parser:    The parser used to raise an error if one occurs
    @param options:   The possible options that each input can take. If no value is assigned to a particular option, then it is assigned None
    @param compulsary_args:   The exact number of compulsary arguments (colon separated)
    @return A list of dictionaries containing the parsed arguments
    '''
    input_dicts = []

    for input_string in inputs:
        split_input = input_string.split(":")
        if len(split_input) < compulsary_args:
            parser.error("A mandatory data group option is missing")
        elif len(split_input) > compulsary_args+1:
            parser.error("Too many mandatory data groups")

        input_dict = {}

        option = options._asdict().keys()

        # First deal with the comma separated compulsary arguments
        for i in range(0, compulsary_args):
            try:
                current_option = split_input[i]
                input_dict[option[0]] = options[i](current_option, parser)
                option.pop(0)  # Compulsary arguments always the first in the list
            except IndexError:
                input_dict[option[i]] = None

        # Now deal with optional arugments, if they exist. For each option loop through the list of arguments to see if
        # it exists, if so check and add to the dictionary.
        if len(split_input) == compulsary_args+1:
            split_input_comma = split_input[-1].split(",")
        else:
            split_input_comma = []  # need to loop over options to set optional arguments to None

        for i, option in enumerate(option):
            # Make sure an entry for each option is created, even if it is None
            input_dict[option] = None
            for j in split_input_comma:
                    # Split the input, [0] will be the key and [1] the value in the list
                    split_input_variable = j.split("=")
                    if split_input_variable[0] == option:
                        input_dict[option] = options[i+compulsary_args](split_input_variable[1], parser)

        input_dicts.append(input_dict)
    return input_dicts


def split_outside_brackets(input, seps=[','], brackets={'[': ']'}):
    """Splits an input string at separators that are not within brackets.
    @param input: input string to parse
    @param seps: list of separator characters - default: comma
    @param brackets: map of open brackets to corresponding close brackets: default: square brackets
    @return: list of strings formed by breaking the input at colons
    """
    match_brackets = []
    match_bracket = None
    in_brackets = 0
    sep_idxs = []
    for idx in range(0, len(input)):
        if input[idx] in brackets:
            if in_brackets > 0:
                match_brackets.append(match_bracket)
            match_bracket = brackets[input[idx]]
            in_brackets += 1
        elif in_brackets > 0 and input[idx] == match_bracket:
            in_brackets -= 1
            if in_brackets > 0:
                match_bracket = match_brackets.pop()
        elif input[idx] in seps and in_brackets == 0:
            sep_idxs.append(idx)

    output = []
    start_idx = 0
    for idx in sep_idxs:
        output.append(input[start_idx:idx])
        start_idx = idx + 1
    if start_idx < len(input):
        output.append(input[start_idx:])
    return output


def extract_method_and_args(arguments, parser):
    from jasmin_cis.utils import parse_key_val_list
    if not arguments:
        method_and_args = None
    else:
        elements = arguments.split(',')
        method_name = elements[0]
        args = elements[1:] if len(elements) > 1 else []
        method_and_args = ( method_name, parse_key_val_list(args) )
    return method_and_args


def check_nothing(item, parser):
    return item


def check_is_not_empty(item, parser):
    if not item:
        parser.error("Non optional argument not specified in datagroup")
    return item



def check_plot_type(plot_type, datagroups, parser):
    '''
    Checks plot type is valid option for number of variables if specified
    '''

    if plot_type is not None:
        if plot_type not in Plotter.plot_types.keys():
            parser.error("'" + plot_type + "' is not a valid plot type, please use one of: " + str(Plotter.plot_types.keys()))

def check_color(color, parser):
    if color:
        from matplotlib.colors import cnames
        color = color.lower()
        if (color not in cnames) and color != "grey":
            parser.error("'" + color + "' is not a valid colour")
    else:
        color = None
    return color


def check_colour_bar_orientation(orientation, parser):
    orientation = orientation.lower()
    if orientation != "horizontal" and orientation != "vertical":
        parser.error("The colour bar orientation must either be horizontal or vertical")
    return orientation

def parse_as_float_or_date(arg, name, parser):
    from time_util import parse_datetimestr_to_std_time
    if arg:
        try:
            # First try and parse as a float
            arg = float(arg)
        except ValueError:
            # Then try and parse as a date
            try:
                arg = parse_datetimestr_to_std_time(arg)
            except ValueError:
                # Otherwise throw an error
                parser.error("'" + arg + "' is not a valid " + name)
        return arg
    else:
        return None

def parse_as_float_or_time_delta(arg, name, parser):
    from time_util import parse_datetimestr_delta_to_float_days
    if arg:
        try:
            # First try and parse as a float
            arg = float(arg)
        except ValueError:
            # Then try and parse as a timedelta
            try:
                arg = parse_datetimestr_delta_to_float_days(arg)
            except ValueError:
                # Otherwise throw an error
                parser.error("'" + arg + "' is not a valid " + name)
        return arg
    else:
        return None

def check_range(min_val, max_val, step, parser, range_axis):
    '''
    If a val range was specified, checks that they are valid numbers and the min is less than the max
    '''
    ax_range = {}

    if min_val is not None:
        ax_range[range_axis + "min"] = parse_as_float_or_date(min_val, range_axis + "min", parser)

    if max_val is not None:
        ax_range[range_axis + "max"] = parse_as_float_or_date(max_val, range_axis + "max", parser)

    if ax_range.get(range_axis + "min", None) is not None and ax_range.get(range_axis + "max", None) is not None:
        if ax_range[range_axis + "min"] >= ax_range[range_axis + "max"]:
            parser.error(range_axis + "min must be less than " + range_axis + "max")

    if step is not None:
        ax_range[range_axis + "step"] = parse_as_float_or_time_delta(step, range_axis + "step", parser)

    return ax_range


def check_boolean_argument(argument):
    if argument is None or argument != "False":
        return True
    else:
        return False


def assign_logs(arguments):
    arguments.logx = check_boolean_argument(arguments.logx)
    arguments.logy = check_boolean_argument(arguments.logy)
    arguments.logv = check_boolean_argument(arguments.logv)
    
    if arguments.logx:
        arguments.logx = 10
    else:
        arguments.logx = None
        
    if arguments.logy:
        arguments.logy = 10
    else:
        arguments.logy = None

    if arguments.logv:
        arguments.logv = 10
    else:
        arguments.logv = None
    
    return arguments

def validate_plot_args(arguments, parser):
    arguments.datagroups = get_plot_datagroups(arguments.datagroups, parser)
    check_plot_type(arguments.type, arguments.datagroups, parser)

    arguments.valrange = check_range(arguments.vmin, arguments.vmax, arguments.vstep, parser, "v")
    arguments.xrange = check_range(arguments.xmin, arguments.xmax, arguments.xstep, parser, "x")
    arguments.yrange = check_range(arguments.ymin, arguments.ymax, arguments.ystep, parser, "y")

    arguments.cbarorient = check_colour_bar_orientation(arguments.cbarorient, parser)
    arguments.nocolourbar = check_boolean_argument(arguments.nocolourbar)
    arguments.grid = check_boolean_argument(arguments.grid)

    arguments.coastlinescolour = check_color(arguments.coastlinescolour, parser)

    arguments = assign_logs(arguments)
    # Try and parse numbers
    arguments.itemwidth = parse_float(arguments.itemwidth, "item width", parser)   
    arguments.fontsize = parse_float(arguments.fontsize, "font size", parser)
    arguments.height = parse_float(arguments.height, "height", parser)
    arguments.width = parse_float(arguments.width, "width", parser)
    arguments.xtickangle = parse_float(arguments.xtickangle, "x tick angle", parser)
    arguments.ytickangle = parse_float(arguments.ytickangle, "y tick angle", parser)
    arguments.xbinwidth = parse_float(arguments.xbinwidth, "x bin width", parser)
    arguments.ybinwidth = parse_float(arguments.ybinwidth, "y bin width", parser)
    
    return arguments


def validate_info_args(arguments, parser):
    check_file_exists(arguments.filename, parser)
    return arguments


def validate_col_args(arguments, parser):
    '''
    Checks that the filenames are valid and that variables and methods have been specified.
    Assigns default method/variable to datagroups with unspecified method/variable if default is specified
    Checks that the product is valid if specified
    '''
    # Note: Sample group is put into a list as parse_colonic_arguments expects a list. Samplegroup will only ever be one argument though
    samplegroup = get_col_samplegroup([arguments.samplegroup], parser)

    # Take the three parts out of the 0th samplegroup. Note: Due to the reason stated above, there will only ever be one samplegroup
    arguments.samplefiles = samplegroup[0]["filenames"]
    arguments.samplevariable = samplegroup[0]["variable"] if samplegroup[0]["variable"] is not "" else None
    arguments.sampleproduct = samplegroup[0]["product"]
    arguments.datagroups = get_col_datagroups(arguments.datagroups, parser)

    return arguments


def validate_subset_args(arguments, parser):
    arguments.datagroups = get_subset_datagroups(arguments.datagroups, parser)
    arguments.limits = get_subset_limits(arguments.subsetranges, parser)
    return arguments


def validate_version_args(arguments, parser):
    # no arguments accepted
    return arguments


validators = {'plot': validate_plot_args,
              'info': validate_info_args,
              'col': validate_col_args,
              'subset': validate_subset_args,
              'version': validate_version_args}


def parse_args(arguments = None):
    '''
    Parse the arguments given. If no arguments are given, then used the command line arguments.
    Returns a dictionary contains the parsed arguments
    '''
    parser = initialise_top_parser()
    if arguments is None:
        #sys.argv[0] is the name of the script itself
        arguments = sys.argv[1:]
    main_args = parser.parse_args(arguments)
    main_args = validators[main_args.command](main_args, parser)

    return main_args



"""
Module containing hdf file utility functions for the SD object
"""
import logging
from jasmin_cis.utils import create_masked_array_for_missing_values


def get_hdf_SD_file_variables(filename):
    '''
    Get all the variables from an HDF SD file

    args:
        filename: The filename of the file to get the variables from

    returns:
        An OrderedDict containing the variables from the file
    '''
    from pyhdf import SD

    variables = None

    try:
        # Open the file.
        datafile = SD.SD(filename)
        # List of required variable names.
        variables = datafile.datasets()
        # Close the file
        datafile.end()
    except:
        logging.error("Error while reading SD data")

    return variables


def read(filename, variables=None, datadict=None):
    """
    Reads SD from a HDF4 file into a dictionary. 
    
    Returns:
        A dictionary containing data for requested variables.
        Missing data is replaced by NaN.
        
    Arguments:
        filename    -- The name (with path) of the HDF file to read.
        names       -- A sequence of variable (dataset) names to read from the
                       file (default None, causing all variables to be read).
                       The names must appear exactly as in in the HDF file.
        datadict     -- Optional dictionary to add data to, otherwise a new, empty
                       dictionary is created
    
    """
    from pyhdf import SD

    # Open the file.
    datafile = SD.SD(filename)


    # List of required variable names.
    if variables is None:
        variables = datafile.datasets().keys()
    if not isinstance(variables,list): variables = [ variables ]

    # Create dictionary to hold data arrays for returning.
    if datadict is None:
        datadict = {}

    # Get data.
    for variable in variables:
        try:
            sds = datafile.select(variable) # SDS object.
            datadict[variable] = sds
        except:
            # ignore variable that failed
            pass
    
    return datadict


def get_calipso_data(sds):
    """
    Reads raw data from an SD instance. Automatically applies the
    scaling factors and offsets to the data arrays found in Calipso data.

    Returns:
        A numpy array containing the raw data with missing data is replaced by NaN.

    Arguments:
        sds        -- The specific sds instance to read

    """
    from jasmin_cis.utils import create_masked_array_for_missing_data

    calipso_fill_values = {'Float_32' : -9999.0,
                           #'Int_8' : 'See SDS description',
                           'Int_16' : -9999,
                           'Int_32' : -9999,
                           'UInt_8' : -127,
                           #'UInt_16' : 'See SDS description',
                           #'UInt_32' : 'See SDS description',
                           'ExtinctionQC Fill Value' : 32768,
                           'FeatureFinderQC No Features Found' : 32767,
                           'FeatureFinderQC Fill Value' : 65535}

    data = sds.get()
    attributes = sds.attributes()

    # Missing data.
    missing_val = attributes.get('fillvalue', None)
    if missing_val is None:
        try:
            missing_val = calipso_fill_values[attributes.get('format', None)]
        except KeyError:
            # Last guess
            missing_val = attributes.get('_FillValue', None)

    data = create_masked_array_for_missing_data(data, missing_val)

    # Offsets and scaling.
    offset  = attributes.get('add_offset', 0)
    scale_factor = attributes.get('scale_factor', 1)
    data = __apply_scaling_factor_CALIPSO(data, scale_factor, offset)

    return data


def get_data(sds, missing_values=None):
    """
    Reads raw data from an SD instance. Automatically applies the
    scaling factors and offsets to the data arrays often found in NASA HDF-EOS
    data (e.g. MODIS)

    Returns:
        A numpy array containing the raw data with missing data is replaced by NaN.

    Arguments:
        sds        -- The specific sds instance to read

    """
    from jasmin_cis.utils import create_masked_array_for_missing_data

    data = sds.get()
    attributes = sds.attributes()

    # Missing data.
    if missing_values is None:
        missing_values = [attributes.get('_FillValue', None)]

    data = create_masked_array_for_missing_values(data,missing_values)

    # Offsets and scaling.
    offset  = attributes.get('add_offset', 0)
    scale_factor = attributes.get('scale_factor', 1)
    data = __apply_scaling_factor_MODIS(data, scale_factor, offset)

    return data


def get_metadata(sds):
    from jasmin_cis.data_io.ungridded_data import Metadata

    name = sds.info()[0]
    long_name = sds.attributes().get('long_name',None)
    shape = sds.info()[2]
    units = sds.attributes().get('units')
    valid_range = sds.attributes().get('valid_range')
    factor = sds.attributes().get('scale_factor')
    offset = sds.attributes().get('add_offset')
    missing = sds.attributes().get('_FillValue')

    # put the whole dictionary of attributes into 'misc'
    # so that other metadata of interest can still be retrieved if need be
    misc = sds.attributes()

    metadata = Metadata( name=name, long_name=long_name, shape=shape, units=units, range=valid_range,
        factor=factor, offset=offset, missing_value=missing, misc=misc)


    return metadata


def __apply_scaling_factor_CALIPSO(data, scale_factor, offset):
    '''
    Apply scaling factor Calipso data
    @param data:
    @param scale_factor:
    @param offset:
    @return:
    '''

    data = (data/scale_factor) + offset
    return data


def __apply_scaling_factor_MODIS(data, scale_factor, offset):
    '''
    Apply scaling factor for MODIS data
    @param data:
    @param scale_factor:
    @param offset:
    @return:
    '''
    data = (data - offset) * scale_factor
    return data




'''
Module for finding files
'''
from glob import glob
import numpy as np

def find_satelitte_data(folder,day,year,variable,orbits=None): 
    '''
    Returns a list of filenames from a given folder,
    given a day, year, variable and (optionally) orbits
    for general satellite Level2 data files. 
    '''   
    day = str(day).rjust(3,'0')
    
    if orbits is None:
        filenames = glob(folder + str(year) + '/' + str(day) + '/*')
        filenames = np.sort(filenames)
    else:
        filenames = []
        for orbit in orbits:
            filenames.append(glob(folder + str(year) + '/' + str(day) + '/*_' + str(orbit) + '_*.hdf')[0])
    
    return filenames



from jasmin_cis.plotting.generic_plot import Generic_Plot
from jasmin_cis.plotting.heatmap import Heatmap
from jasmin_cis.plotting.scatter_plot import Scatter_Plot

class Scatter_Overlay(Generic_Plot):
    def plot(self):
        '''
        Plots a heatmap overlayed with one or more scatter plots
        '''
        from jasmin_cis.exceptions import InvalidNumberOfDatagroupsSpecifiedError
        if len(self.packed_data_items) > 1:
            Heatmap([self.packed_data_items[0]], self.plot_args, *self.mplargs, **self.mplkwargs)
            scatter_plot_args = self.plot_args
            scatter_plot_args["datagroups"] = self.plot_args["datagroups"][1:]
            self.scatter_plots = Scatter_Plot(self.packed_data_items[1:], scatter_plot_args, calculate_min_and_max_values = False, *self.mplargs, **self.mplkwargs)
        else:
            raise InvalidNumberOfDatagroupsSpecifiedError("Scatter overlay requires two or more datagroups")

    def set_axis_label(self, axis, options):
        import jasmin_cis.exceptions as cisex
        import iris.exceptions as irisex
        axis = axis.lower()
        axislabel = axis + "label"

        if options[axislabel] is None:
            if self.is_map():
                options[axislabel] = "Longitude" if axis == "x" else "Latitude"
            else:
                try:
                    name = self.packed_data_items[0].coord(name=self.plot_args[axis + "_variable"]).name()
                except (cisex.CoordinateNotFoundError, irisex.CoordinateNotFoundError):
                    name = self.packed_data_items[0].name()

                try:
                    units = self.packed_data_items[0].coord(name=self.plot_args[axis + "_variable"]).units
                except (cisex.CoordinateNotFoundError, irisex.CoordinateNotFoundError):
                    units = self.packed_data_items[0].units

                # in general, display both name and units in brackets
                options[axislabel] = name + " " + self.format_units(units)

        return options

    def create_legend(self):
        self.scatter_plots.create_legend()

    def format_plot(self):
        self.format_time_axis()
        self.format_3d_plot()

    def set_default_axis_label(self, axis):
        self.set_3daxis_label(axis)


import numpy.ma as ma
import numpy as np

first_array = ma.array([5,-1,5],mask=[0,1,0])
second_array = ma.array([8,9,-1],mask=[0,0,1])

intersection = ma.mask_or(first_array.mask, second_array.mask)

new_array = ma.array(first_array, mask = intersection)

print intersection
print new_array


'''
Module to test the one-dimensional plotting of NetCDF files
More tests can be found in the manual_integration_tests package
'''
import os.path

from nose.tools import istest, raises
from jasmin_cis.test.test_files.data import *
import iris
import os.path

from jasmin_cis.plotting.plot import Plotter


def delete_file_if_exists():
    '''
    Used to delete the file that will be created before tests are run
    in order to be able to check after the test if the file was created by the test
    '''
    if os.path.isfile(out_filename):
        os.remove(out_filename)

def make_cube(filename, variable = None):
    if variable is None:
        variable = valid_variable_in_valid_filename
    variable = iris.AttributeConstraint(name = variable)
    cube = iris.load_cube(filename, variable) 
    cube = list(cube.slices([ coord for coord in cube.coords() if coord.points.size > 1]))[0]
    return cube       
    
@istest
@raises(IOError)
def should_raise_io_error_with_invalid_filename():     
    cube = make_cube("/")
    Plotter([cube], "line", "/")



'''
module to test the NetCDF module
'''
from netCDF4 import Variable
from nose.tools import istest,raises, eq_
from jasmin_cis.test.test_files.data import *
from jasmin_cis.data_io.netcdf import *

@istest
@raises(IOError)
def should_raise_io_error_with_file_that_is_not_netcdf():
    read(valid_aeronet_filename, valid_aeronet_variable)

@istest
def test_that_can_read_all_variables():
    dict = get_netcdf_file_variables(valid_2d_filename)
    eq_(len(dict),426)

@istest
def test_that_can_read_known_variable():
    data = read(valid_2d_filename,'latitude')
    assert(isinstance(data['latitude'],Variable))

@istest
def test_that_can_get_data():
    data = read(valid_2d_filename, 'latitude')
    eq_(data['latitude'].shape,(145,))

@istest
def test_that_can_get_metadata_for_known_variable():
    data = read(valid_2d_filename,'rain')
    metadata = get_metadata(data['rain'])

    eq_(str(metadata.missing_value),"2e+20")
    eq_(metadata.long_name,"TOTAL RAINFALL RATE: LS+CONV KG/M2/S")
    eq_(metadata.units,"kg m-2 s-1")

@istest
@raises(IOError)
def should_raise_ioerror_with_file_that_does_not_have_read_permissions():
    read(file_without_read_permissions, valid_variable_in_valid_filename)




'''
    Top level colocation object
'''
import logging


class Colocate(object):

    def __init__(self, sample_files, sample_var, sample_product, output_file):
        from jasmin_cis.data_io.read import read_coordinates, read_data
        from jasmin_cis.data_io.write_netcdf import write_coordinates

        self.sample_files = sample_files
        if sample_var is None:
            coords = read_coordinates(sample_files, sample_product)
            sample_points = coords.get_coordinates_points()

        else:
            data = read_data(sample_files, sample_var, sample_product)
            coords = data.coords()
            sample_points = data.get_all_points()

        write_coordinates(coords, output_file)

        self.sample_points = sample_points
        self.output_file = output_file

    @staticmethod
    def _get_valid_colocator_instance(col_name, col_params):
        from jasmin_cis.col_framework import get_colocator
        from jasmin_cis.exceptions import InvalidCommandLineOptionError

        if col_name is None:
            col_name = 'DefaultColocator'
        col_cls = get_colocator(col_name)

        try:
            if col_params is not None:
                col = col_cls(**col_params)
            else:
                col = col_cls()
        except TypeError as e:
            raise InvalidCommandLineOptionError(str(e)+"\nInvalid argument for specified colocator.")
        return col

    @staticmethod
    def _get_valid_constraint_instance(con_name, con_params):
        from jasmin_cis.col_framework import get_constraint
        from jasmin_cis.exceptions import InvalidCommandLineOptionError

        if con_name is None:
            con_name = 'DummyConstraint'
        con_cls = get_constraint(con_name)

        try:
            if con_params is not None:
                con = con_cls(**con_params)
            else:
                con = con_cls()
        except TypeError as e:
            raise InvalidCommandLineOptionError(str(e)+"\nInvalid argument for specified constraint method.")
        return con

    @staticmethod
    def _get_valid_kernel_instance(kern_name, kern_params, cube=False):
        from jasmin_cis.col_framework import get_kernel
        from jasmin_cis.exceptions import InvalidCommandLineOptionError

        if kern_name is None:
            if cube:
                kern_name = 'nn_gridded'
            else:
                kern_name = 'nn_horizontal'
        kern_cls = get_kernel(kern_name)

        try:
            if kern_params is not None:
                kernel = kern_cls(**kern_params)
            else:
                kernel = kern_cls()
        except TypeError as e:
            raise InvalidCommandLineOptionError(str(e)+"\nInvalid argument for specified kernel.")

        return kernel

    def colocate(self, variable, filenames, col_name=None, col_params=None, con_method=None, con_params=None, kern=None, kern_params=None, product = None):
        from jasmin_cis.data_io.read import read_data
        from jasmin_cis.data_io.write_netcdf import add_data_to_file
        from jasmin_cis.exceptions import CoordinateNotFoundError
        from time import time
        from iris import cube
        from jasmin_cis.cis import __version__

        logging.info("Reading data for: "+variable)
        data = read_data(filenames, variable, product)

        # Find colocator, constraint_fn and kernel to use
        col = Colocate._get_valid_colocator_instance(col_name, col_params)
        con = Colocate._get_valid_constraint_instance(con_method, con_params)
        kernel = Colocate._get_valid_kernel_instance(kern, kern_params, isinstance(data, cube.Cube))

        logging.info("Colocator: ", col_name)
        logging.info("Constraints: ", con_method)
        logging.info("kernel: ", kern)

        logging.info("Colocating, this could take a while...")
        t1 = time()
        try:
            new_data = col.colocate(self.sample_points, data, con, kernel)
        except TypeError as e:
            raise CoordinateNotFoundError('Colocator was unable to compare data points, check the dimensions of each '
                                          'data set and the co-location methods chosen. \n'+str(e))

        logging.info("Completed. Total time taken: " + str(time()-t1))

        logging.info("Appending data to "+self.output_file)
        for data in new_data:
            data.metadata.history += "Colocated onto sampling from: " + str(self.sample_files) + " using CIS version " + __version__ + \
                                      "\nvariable: " + str(variable) + \
                                      "\nwith files: " + str(filenames) + \
                                      "\nusing colocator: " + str(col_name) + \
                                      "\ncolocator parameters: " + str(col_params) + \
                                      "\nconstraint method: " + str(con_method) + \
                                      "\nconstraint parameters: " + str(con_params) + \
                                      "\nkernel: " + str(kern) + \
                                      "\nkernel parameters: " + str(kern_params)

            add_data_to_file(data, self.output_file)


from abc import ABCMeta
import logging
from collections import namedtuple

import numpy as np
import iris

from jasmin_cis.subsetting.subset_framework import SubsetConstraintInterface
from jasmin_cis.data_io.Coord import Coord, CoordList
from jasmin_cis.data_io.ungridded_data import LazyData, UngriddedData


class CoordLimits(namedtuple('CoordLimits', ['coord', 'start', 'end', 'constraint_function'])):
    """Holds the start and end values for subsetting limits.
    @ivar coord: the coordinate the limit applies to
    @ivar start: subsetting limit start
    @ivar end: subsetting limit end
    @ivar constraint_function: function determining whether the constraint is satisfied
    """
    pass


class SubsetConstraint(SubsetConstraintInterface):
    """Abstract Constraint for subsetting.

    Holds the limits for subsetting in each dimension.
    """
    __metaclass__ = ABCMeta

    def __init__(self):
        self._limits = {}
        logging.debug("Created SubsetConstraint of type %s", self.__class__.__name__)

    def set_limit(self, coord, dim_min, dim_max):
        """Sets boundary values for a dimension to be used in subsetting.

        @param coord: coordinate to which limit applies
        @param dim_min: lower bound on dimension or None to indicate no lower bound
        @param dim_max: upper bound on dimension or None to indicate no upper bound
        """
        if dim_min is not None or dim_max is not None:
            logging.info("Setting limit for dimension '%s' [%s, %s]", coord.name(), str(dim_min), str(dim_max))
            self._limits[coord.name()] = CoordLimits(coord, dim_min, dim_max,
                                                     self._make_constraint_function(dim_min, dim_max))

    @staticmethod
    def _make_constraint_function(dim_min, dim_max):
        """Constructs a function enforcing the specified bounds on the values of a dimension.

        The boundary values are included in the constrained interval.
        @param dim_min: lower bound on dimension or None to indicate no lower bound
        @param dim_max: upper bound on dimension or None to indicate no upper bound
        @return: lambda function with one argument returning bool
        """
        if dim_min is not None and dim_max is not None:
            return lambda x: dim_min <= x <= dim_max
        elif dim_min is not None and dim_max is None:
            return lambda x: dim_min <= x
        elif dim_min is None and dim_max is not None:
            return lambda x: x <= dim_max
        else:
            return None

    def __str__(self):
        limit_strs = []
        for name, limit in self._limits.iteritems():
            limit_strs.append("{}: [{}, {}]".format(name, str(limit.start), str(limit.end)))
        return ', '.join(limit_strs)


class GriddedSubsetConstraint(SubsetConstraint):
    """Implementation of SubsetConstraint for subsetting gridded data.
    """
    def make_iris_constraint(self):
        """Constructs an Iris constraint corresponding to the limits set for each dimension.

        @return: iris.Constraint object
        """
        constraint = None
        for coord, limits in self._limits.iteritems():
            constraint_function = limits.constraint_function
            if constraint is None:
                constraint = iris.Constraint(**{coord: constraint_function})
            else:
                constraint = constraint & iris.Constraint(**{coord: constraint_function})
        return constraint

    def constrain(self, data):
        """Subsets the supplied data.

        @param data: data to be subsetted
        @type data: iris.cube.Cube
        @return: subsetted data
        @rtype: iris.cube.Cube
        """
        iris_constraint = self.make_iris_constraint()
        return data.extract(iris_constraint)


class UngriddedSubsetConstraint(SubsetConstraint):
    """Implementation of SubsetConstraint for subsetting ungridded data.
    """
    def constrain(self, data):
        """Subsets the supplied data.

        @param data: data to be subsetted
        @return: subsetted data
        """
        CoordPair = namedtuple('CoordPair', ['input', 'output'])

        new_values = []
        coord_pairs = []
        for coord in data.coords():
            coord_pairs.append(CoordPair(coord, []))

        # Filter points to include in subset.
        is_masked = isinstance(data.data, np.ma.masked_array)
        missing_value = data.metadata.missing_value
        for idx, value in enumerate(data.data.flat):
            if is_masked and value is np.ma.masked:
                value = missing_value
                # continue

            include = True
            for limit in self._limits.itervalues():
                if not limit.constraint_function(limit.coord.data_flattened[idx]):
                    include = False
                    break
            if include:
                # Append coordinates and value.
                new_values.append(value)
                for coord in coord_pairs:
                    coord.output.append(coord.input.data_flattened[idx])

        # Collect output into object.
        new_data = np.array(new_values, dtype=data.data.dtype, copy=False)
        subset_metadata = data.metadata
        subset_metadata.shape = (len(new_values),)

        subset_coords = CoordList()
        for coord in coord_pairs:
            new_coord = Coord(np.array(coord.output, dtype=coord.input.data.dtype, copy=False), coord.input.metadata)
            new_coord.metadata.shape = (len(coord.output),)
            subset_coords.append(new_coord)
        subset = UngriddedData(new_data, subset_metadata, subset_coords)

        return subset


class UngriddedOnGridSubsetConstraint(SubsetConstraint):
    """Implementation of SubsetConstraint for subsetting data held in
    an UngriddedData object but which has coordinates on a grid.
    """
    def constrain(self, data):
        """Subsets the supplied data.

        @param data: data to be subsetted
        @return: subsetted data
        """
        shape = data.data.shape
        ndim = len(shape)
        #TODO Coordinates should have the same shape.

        # Initialise arrays for new values and slices that are included in the subset.
        min_indices = list(shape)
        max_indices = [0] * ndim

        # Filter points to include in subset.
        for idx in _index_iterator(shape):
            include = True
            for limit in self._limits.itervalues():
                if not limit.constraint_function(limit.coord.data[idx]):
                    include = False
                    break
            if include:
                # Append coordinates and value.
                for j in range(ndim):
                    if idx[j] > max_indices[j]:
                        max_indices[j] = idx[j]
                    if idx[j] < min_indices[j]:
                        min_indices[j] = idx[j]

        # Construct slices and shape representing the region allowed by the constraints.
        slice_list = []
        shape_list = []
        num_values = 1
        for j in range(ndim):
            slice_list.append(slice(min_indices[j], max_indices[j]))
            shape_list.append(max_indices[j] - min_indices[j])
            num_values *= (max_indices[j] - min_indices[j])
        slice_tuple = tuple(slice_list)
        shape_tuple = tuple(shape_list)

        if num_values > 0:
            # Collect output into object.
            new_data = np.array(data.data[slice_tuple], copy=False)
            subset_metadata = data.metadata
            subset_metadata.shape = shape_tuple

            subset_coords = CoordList()
            for coord in data.coords():
                coord_metadata = coord.metadata
                coord_metadata.shape = shape_tuple
                new_coord = Coord(np.array(coord.data[slice_tuple], copy=False), coord_metadata)
                subset_coords.append(new_coord)
            subset = UngriddedData(new_data, subset_metadata, subset_coords)
        else:
            subset = None
        return subset


def _index_iterator(shape):
    """Iterates over the indexes of a multi-dimensional array of a specified shape.

    The last index changes most rapidly.
    @param shape: sequence of array dimensions
    @return: yields tuples of array indexes
    """
    dim = len(shape)
    idx = [0] * dim
    num_iterations = 1
    for j in range(0, dim):
        num_iterations *= shape[j]

    for iterations in range(num_iterations):
        yield tuple(idx)
        for j in range(dim - 1, -1, -1):
            idx[j] += 1
            if idx[j] < shape[j]:
                break
            idx[j] = 0



from jasmin_cis.plotting.generic_plot import Generic_Plot

class Histogram_2D(Generic_Plot):
    valid_histogram_styles = ["bar", "step", "stepfilled"]

    def plot(self):
        '''
        Plots a 2D histogram
        '''
        from numpy.ma import MaskedArray
        vmin = self.mplkwargs.pop("vmin")
        vmax = self.mplkwargs.pop("vmax")

        self.mplkwargs["bins"] = self.calculate_bin_edges()

        for i, unpacked_data_item in enumerate(self.unpacked_data_items):
            datafile = self.plot_args["datagroups"][i]
            if datafile["itemstyle"]:
                if datafile["itemstyle"] in self.valid_histogram_styles:
                    self.mplkwargs["histtype"] = datafile["itemstyle"]
                else:
                    from jasmin_cis.exceptions import InvalidHistogramStyleError
                    raise InvalidHistogramStyleError("'" + datafile["itemstyle"] + "' is not a valid histogram style, please use one of: " + str(self.valid_histogram_styles))

            else:
                self.mplkwargs.pop("histtype", None)

            if datafile["color"]:
                self.mplkwargs["color"] = datafile["color"]
            else:
                self.mplkwargs.pop("color", None)

            if isinstance(unpacked_data_item["data"], MaskedArray):
                data = unpacked_data_item["data"].compressed()
            else:
                data = unpacked_data_item["data"].flatten()

            self.matplotlib.hist(data, *self.mplargs, **self.mplkwargs)
        self.mplkwargs["vmin"] = vmin
        self.mplkwargs["vmax"] = vmax

    def unpack_data_items(self):
        return self.unpack_comparative_data()

    def set_plotting_library(self):
        pass

    def calculate_bin_edges(self):
        '''
        Calculates the number of bins for a given axis.
        Uses 10 as the default
        @param axis: The axis to calculate the number of bins for
        @return: The number of bins for the given axis
        '''
        from jasmin_cis.utils import calculate_histogram_bin_edges
        from numpy import array
        min_val = min(unpacked_data_item["data"].min() for unpacked_data_item in self.unpacked_data_items)
        max_val = max(unpacked_data_item["data"].max() for unpacked_data_item in self.unpacked_data_items)
        data = array([min_val, max_val])
        bin_edges = calculate_histogram_bin_edges(data, "x", self.plot_args["xrange"], self.plot_args["xbinwidth"], self.plot_args["logx"])

        self.plot_args["xrange"]["xmin"] = bin_edges.min()
        self.plot_args["xrange"]["xmax"] = bin_edges.max()

        return bin_edges


    def format_plot(self):
        # We don't format the time axis here as we're only plotting frequency against data
        self.format_2d_plot()

    def set_default_axis_label(self, axis):
        '''
        Sets the default axis label for the given axis.
        If the axis is "y", then labels it "Frequency", else works it out based on the name and units of the data
        @param axis: The axis to calculate the default axis label for
        '''
        axis = axis.lower()
        axislabel = axis + "label"

        if self.plot_args[axislabel] is None:
            if axis == "x":
                units = self.packed_data_items[0].units


                if len(self.packed_data_items) == 1:
                    name = self.packed_data_items[0].name()
                    # only 1 data to plot, display
                    self.plot_args[axislabel] = name + " " + self.format_units(units)
                else:
                    # if more than 1 data, legend will tell us what the name is. so just displaying units
                    self.plot_args[axislabel] = self.format_units(units)
            elif axis == "y":
                self.plot_args[axislabel] = "Frequency"

    def calculate_axis_limits(self, axis, min_val, max_val, step):
        '''
        Calculates the limits for a given axis.
        If the axis is "y" then looks at the "data" as this is where the values are stored
        @param axis: The axis to calculate the limits for
        @return: A dictionary containing the min and max values for the given axis
        '''
        if axis == "x":
            coord_axis = "x"
        elif axis == "y":
            coord_axis = "data"
        calculated_min, calculated_max = self.calculate_min_and_max_values_of_array_including_case_of_log(axis, self.unpacked_data_items[0][coord_axis])

        valrange = {}
        valrange[axis + "min"] = calculated_min if min_val is None else min_val
        valrange[axis + "max"] = calculated_max if max_val is None else max_val
        valrange[axis + "step"] = step

        return valrange

    def set_axis_ticks(self, axis, no_of_dims):

        if axis == "x":
            tick_method = self.matplotlib.xticks
        elif axis == "y":
            tick_method = self.matplotlib.yticks

        if self.plot_args.get(axis + "tickangle", None) is None:
            angle = None
        else:
            angle = self.plot_args[axis + "tickangle"]

        tick_method(rotation=angle)

    def apply_axis_limits(self, valrange, axis):
        '''
        Applies the limits (if given) to the specified axis.
        Sets the "y" axis as having a minimum value of 0 as can never have a negative frequency
        @param valrange: A dictionary containing the min and/or max values for the axis
        @param axis: The axis to apply the limits to
        '''
        if len(valrange) != 0:
            if axis == "x":
                step = valrange.pop("xstep", None)
                self.matplotlib.xlim(**valrange)
                if step is not None: valrange["xstep"] = step
            elif axis == "y":
                step = valrange.pop("ystep", None)
                self.matplotlib.ylim(**valrange)
                if step is not None: valrange["ystep"] = step
        elif axis == "y":
            # Need to ensure that frequency starts from 0
            self.matplotlib.ylim(ymin = 0)





'''
Module for creating mock, dummies and fakes
'''

def make_dummy_2d_cube():
    '''
        Makes a dummy cube filled with random datapoints of shape 19x36
    '''
    import numpy
    from iris.cube import Cube
    from iris.coords import DimCoord
    
    latitude = DimCoord(range(-85, 105, 10), standard_name='latitude', units='degrees')
    longitude = DimCoord(range(0, 360, 10), standard_name='longitude', units='degrees')
    cube = Cube(numpy.random.rand(19, 36), dim_coords_and_dims=[(latitude, 0), (longitude, 1)])
    
    return cube

def make_square_5x3_2d_cube():
    '''
        Makes a well defined cube of shape 5x3 with data as follows
        array([[1,2,3],
               [4,5,6],
               [7,8,9],
               [10,11,12],
               [13,14,15]])
        and coordinates in latitude:
            array([ -10, -5, 0, 5, 10 ])
        longitude:
            array([ -5, 0, 5 ])
            
        They are different lengths to make it easier to distinguish. Note the latitude increases
        as you step through the array in order - so downwards as it's written above
    '''
    import numpy as np
    from iris.cube import Cube
    from iris.coords import DimCoord
    
    latitude = DimCoord(np.arange(-10, 11, 5), standard_name='latitude', units='degrees')
    longitude = DimCoord(np.arange(-5, 6, 5), standard_name='longitude', units='degrees')
    data = np.reshape(np.arange(15)+1.0,(5,3))
    cube = Cube(data, dim_coords_and_dims=[(latitude, 0), (longitude, 1)])
    
    return cube


def make_square_5x3_2d_cube_with_time():
    '''
        Makes a well defined cube of shape 5x3 with data as follows
        arr([[[   1.    2.    3.    4.    5.    6.    7.]
              [   8.    9.   10.   11.   12.   13.   14.]
              [  15.   16.   17.   18.   19.   20.   21.]]

             [[  22.   23.   24.   25.   26.   27.   28.]
              [  29.   30.   31.   32.   33.   34.   35.]
              [  36.   37.   38.   39.   40.   41.   42.]]

             [[  43.   44.   45.   46.   47.   48.   49.]
              [  50.   51.   52.   53.   54.   55.   56.]
              [  57.   58.   59.   60.   61.   62.   63.]]

             [[  64.   65.   66.   67.   68.   69.   70.]
              [  71.   72.   73.   74.   75.   76.   77.]
              [  78.   79.   80.   81.   82.   83.   84.]]

             [[  85.   86.   87.   88.   89.   90.   91.]
              [  92.   93.   94.   95.   96.   97.   98.]
              [  99.  100.  101.  102.  103.  104.  105.]]])
        and coordinates in latitude:
            array([ -10, -5, 0, 5, 10 ])
        longitude:
            array([ -5, 0, 5 ])
        time:
            array([1984-08-27, 1984-08-28, 1984-08-29, 1984-08-30, 1984-08-31, 1984-09-01, 1984-09-02])

        They are different lengths to make it easier to distinguish. Note the latitude increases
        as you step through the array in order - so downwards as it's written above
    '''
    import numpy as np
    from iris.cube import Cube
    from iris.coords import DimCoord
    import datetime
    from jasmin_cis.time_util import convert_obj_to_standard_date_array

    t0 = datetime.datetime(1984,8,27)
    times = np.array([t0+datetime.timedelta(days=d) for d in xrange(7)])

    time_nums = convert_obj_to_standard_date_array(times)

    time = DimCoord(time_nums, standard_name='time')
    latitude = DimCoord(np.arange(-10, 11, 5), standard_name='latitude', units='degrees')
    longitude = DimCoord(np.arange(-5, 6, 5), standard_name='longitude', units='degrees')
    data = np.reshape(np.arange(105)+1.0,(5,3,7))
    cube = Cube(data, dim_coords_and_dims=[(latitude, 0), (longitude, 1), (time, 2)])

    print data

    return cube


def make_dummy_1d_cube():
    import numpy
    from iris.cube import Cube
    from iris.coords import DimCoord
    
    latitude = DimCoord(range(-85, 105, 10), standard_name='latitude', units='degrees')
    cube = Cube(numpy.random.rand(19), dim_coords_and_dims=[(latitude, 0)])
    
    return cube

def get_random_1d_point():
    '''
        Creates a hyper point at some random point along the Grenwich meridian (lon = 0.0)
    '''
    from jasmin_cis.data_io.hyperpoint import HyperPoint
    return HyperPoint(gen_random_lat())

def get_random_2d_point():
    '''
        Creates a random point on the surface of the globe
    '''
    from jasmin_cis.data_io.hyperpoint import HyperPoint
    return HyperPoint(gen_random_lat(), gen_random_lon())

def get_random_3d_point():
    '''
        Creates a random point in 3d space upto 100km above the surface of the globe
    '''
    import random
    from jasmin_cis.data_io.hyperpoint import HyperPoint
    return HyperPoint(gen_random_lat(), gen_random_lon(), random.randrange(0.0, 100.0))

def make_dummy_1d_points_list(num):
    '''
        Create a list of 1d points 'num' long
    '''
    return [ get_random_1d_point() for i in xrange(0,num) ]

def make_dummy_2d_points_list(num):
    '''
        Create a list of 2d points 'num' long
    '''
    return [ get_random_2d_point() for i in xrange(0,num) ]
        
def make_dummy_1d_ungridded_data():
    from data_io.Coord import CoordList, Coord
    from data_io.ungridded_data import UngriddedData, Metadata

    x = Coord(gen_random_lat_array((5,)), Metadata('latitude'),'x')
    coords = CoordList([x])
    data = gen_random_data_array((5,),4.0,1.0)
    return UngriddedData(data, Metadata(standard_name='rain', long_name="TOTAL RAINFALL RATE: LS+CONV KG/M2/S", units="kg m-2 s-1", missing_value=-999), coords)

def make_dummy_2d_ungridded_data():
    from jasmin_cis.data_io.Coord import CoordList, Coord
    from jasmin_cis.data_io.ungridded_data import UngriddedData, Metadata

    x = Coord(gen_random_lat_array((5,5)), Metadata('latitude'),'x')
    y = Coord(gen_random_lon_array((5,5)), Metadata('longitude'),'y')
    coords = CoordList([x, y])
    data = gen_random_data_array((5,5),4.0,1.0)
    return UngriddedData(data, Metadata(standard_name='rain', long_name="TOTAL RAINFALL RATE: LS+CONV KG/M2/S", units="kg m-2 s-1", missing_value=-999), coords)

def make_regular_2d_ungridded_data():
    '''
        Makes a well defined ungridded data object of shape 5x3 with data as follows
        array([[1,2,3],
               [4,5,6],
               [7,8,9],
               [10,11,12],
               [13,14,15]])
        and coordinates in latitude:
        array([[-10,-10,-10],
               [-5,-5,-5],
               [0,0,0],
               [5,5,5],
               [10,10,10]])
        longitude:
        array([[-5,0,5],
               [-5,0,5],
               [-5,0,5],
               [-5,0,5],
               [-5,0,5]])

        They are different lengths to make it easier to distinguish. Note the latitude increases
        as you step through the array in order - so downwards as it's written above
    '''
    import numpy as np
    from jasmin_cis.data_io.Coord import CoordList, Coord
    from jasmin_cis.data_io.ungridded_data import UngriddedData, Metadata

    x_points = np.arange(-10, 11, 5)
    y_points = np.arange(-5, 6, 5)
    y, x = np.meshgrid(y_points,x_points)

    x = Coord(x, Metadata(standard_name='latitude', units='degrees'))
    y = Coord(y, Metadata(standard_name='longitude', units='degrees'))
    data = np.reshape(np.arange(15)+1.0,(5,3))

    coords = CoordList([x, y])
    return UngriddedData(data, Metadata(standard_name='rain', long_name="TOTAL RAINFALL RATE: LS+CONV KG/M2/S", units="kg m-2 s-1", missing_value=-999), coords)


def make_regular_2d_with_time_ungridded_data():
    '''
        Makes a well defined ungridded data object of shape 5x3 with data as follows
        array([[1,2,3],
               [4,5,6],
               [7,8,9],
               [10,11,12],
               [13,14,15]])
        and coordinates in latitude:
        array([[-10,-10,-10],
               [-5,-5,-5],
               [0,0,0],
               [5,5,5],
               [10,10,10]])
        longitude:
        array([[-5,0,5],
               [-5,0,5],
               [-5,0,5],
               [-5,0,5],
               [-5,0,5]])
        time: np.array( [ 15 day increments from 27th August 1984 ] )
        They are different lengths to make it easier to distinguish. Note the latitude increases
        as you step through the array in order - so downwards as it's written above
    '''
    import numpy as np
    from jasmin_cis.data_io.Coord import CoordList, Coord
    from jasmin_cis.data_io.ungridded_data import UngriddedData, Metadata
    import datetime

    x_points = np.arange(-10, 11, 5)
    y_points = np.arange(-5, 6, 5)
    y, x = np.meshgrid(y_points,x_points)

    t0 = datetime.datetime(1984,8,27)
    times = np.reshape(np.array([t0+datetime.timedelta(days=d) for d in xrange(15)]),(5,3))

    x = Coord(x, Metadata(standard_name='latitude', units='degrees'))
    y = Coord(y, Metadata(standard_name='longitude', units='degrees'))
    t = Coord(times, Metadata(standard_name='time', units='DateTime Object'))

    data = np.reshape(np.arange(15)+1.0,(5,3))

    coords = CoordList([x, y, t])
    return UngriddedData(data, Metadata(standard_name='rain', long_name="TOTAL RAINFALL RATE: LS+CONV KG/M2/S", units="kg m-2 s-1", missing_value=-999), coords)


def make_MODIS_time_steps():
    '''
        Useful for debugging MODIS colocation
    @return:
    '''
    import numpy as np
    from jasmin_cis.data_io.Coord import CoordList, Coord
    from jasmin_cis.data_io.ungridded_data import UngriddedData, Metadata

    x = np.zeros(4)
    times = np.array([149754,149762,149770,149778])

    x = Coord(x, Metadata(standard_name='latitude', units='degrees'))
    t = Coord(times, Metadata(standard_name='time', units='DateTime Object'))

    data = np.arange(4.0)

    coords = CoordList([x, t])
    return UngriddedData(data, Metadata(standard_name='rain', long_name="TOTAL RAINFALL RATE: LS+CONV KG/M2/S", units="kg m-2 s-1", missing_value=-999), coords)


def make_regular_4d_ungridded_data():
    '''
        Makes a well defined ungridded data object of shape 10x5 with data as follows

        data:
        [[  1.   2.   3.   4.   5.]
         [  6.   7.   8.   9.  10.]
         [ 11.  12.  13.  14.  15.]
         [ 16.  17.  18.  19.  20.]
         [ 21.  22.  23.  24.  25.]
         [ 26.  27.  28.  29.  30.]
         [ 31.  32.  33.  34.  35.]
         [ 36.  37.  38.  39.  40.]
         [ 41.  42.  43.  44.  45.]
         [ 46.  47.  48.  49.  50.]]

        latitude:
        [[-10.  -5.   0.   5.  10.]
         [-10.  -5.   0.   5.  10.]
         [-10.  -5.   0.   5.  10.]
         [-10.  -5.   0.   5.  10.]
         [-10.  -5.   0.   5.  10.]
         [-10.  -5.   0.   5.  10.]
         [-10.  -5.   0.   5.  10.]
         [-10.  -5.   0.   5.  10.]
         [-10.  -5.   0.   5.  10.]
         [-10.  -5.   0.   5.  10.]]

        longitude:
        [[-5.  -2.5  0.   2.5  5. ]
         [-5.  -2.5  0.   2.5  5. ]
         [-5.  -2.5  0.   2.5  5. ]
         [-5.  -2.5  0.   2.5  5. ]
         [-5.  -2.5  0.   2.5  5. ]
         [-5.  -2.5  0.   2.5  5. ]
         [-5.  -2.5  0.   2.5  5. ]
         [-5.  -2.5  0.   2.5  5. ]
         [-5.  -2.5  0.   2.5  5. ]
         [-5.  -2.5  0.   2.5  5. ]]

        altitude:
        [[  0.   0.   0.   0.   0.]
         [ 10.  10.  10.  10.  10.]
         [ 20.  20.  20.  20.  20.]
         [ 30.  30.  30.  30.  30.]
         [ 40.  40.  40.  40.  40.]
         [ 50.  50.  50.  50.  50.]
         [ 60.  60.  60.  60.  60.]
         [ 70.  70.  70.  70.  70.]
         [ 80.  80.  80.  80.  80.]
         [ 90.  90.  90.  90.  90.]]

        time:
        [[1984-08-27 1984-08-28 1984-08-29 1984-08-30 1984-08-31]
         [1984-08-27 1984-08-28 1984-08-29 1984-08-30 1984-08-31]
         [1984-08-27 1984-08-28 1984-08-29 1984-08-30 1984-08-31]
         [1984-08-27 1984-08-28 1984-08-29 1984-08-30 1984-08-31]
         [1984-08-27 1984-08-28 1984-08-29 1984-08-30 1984-08-31]
         [1984-08-27 1984-08-28 1984-08-29 1984-08-30 1984-08-31]
         [1984-08-27 1984-08-28 1984-08-29 1984-08-30 1984-08-31]
         [1984-08-27 1984-08-28 1984-08-29 1984-08-30 1984-08-31]
         [1984-08-27 1984-08-28 1984-08-29 1984-08-30 1984-08-31]
         [1984-08-27 1984-08-28 1984-08-29 1984-08-30 1984-08-31]]

        They are shaped to represent a typical lidar type satelite data set.
    '''
    import numpy as np
    from jasmin_cis.data_io.Coord import CoordList, Coord
    from jasmin_cis.data_io.ungridded_data import UngriddedData, Metadata
    import datetime
    from jasmin_cis.time_util import convert_obj_to_standard_date_array

    x_points = np.linspace(-10,10,5)
    y_points = np.linspace(-5, 5, 5)
    t0 = datetime.datetime(1984,8,27)
    times = convert_obj_to_standard_date_array(np.array([t0+datetime.timedelta(days=d) for d in xrange(5)]))

    alt = np.linspace(0,90,10)

    data = np.reshape(np.arange(50)+1.0,(10,5))
    print np.mean(data[:,1:3])
    print np.mean(data[4:6,:])
    print np.mean(data[:,2])
    print np.std(data)
    print np.mean(data)
    print len(data.flat)

    y, a = np.meshgrid(y_points,alt)
    x, a = np.meshgrid(x_points,alt)
    t, a = np.meshgrid(times,alt)

    a = Coord(a, Metadata(standard_name='altitude', units='meters'))
    x = Coord(x, Metadata(standard_name='latitude', units='degrees'))
    y = Coord(y, Metadata(standard_name='longitude', units='degrees'))
    t = Coord(t, Metadata(standard_name='time', units='DateTime Object'))

    coords = CoordList([x, y, a, t])
    return UngriddedData(data, Metadata(standard_name='rain', long_name="TOTAL RAINFALL RATE: LS+CONV KG/M2/S", units="kg m-2 s-1", missing_value=-999), coords)


class ScatterData(object):
    def __init__(self, x, y, data, shape, long_name):
        self.x = x
        self.y = y
        self.data = data
        self.shape = shape
        self.long_name = long_name
        
    def __getitem__(self, name):
        if name == "x":
            return self.x
        elif name == "y":
            return self.y
        elif name == "data":
            return self.data
        else:
            raise Exception("Unknown item")

def gen_random_lon():
    return gen_random_lon_array((1,))

def gen_random_lat():
    return gen_random_lat_array((1,))

def gen_random_data():
    return gen_random_data_array((1,), 0.000225, 0.0001)

def gen_random_lon_array(shape):
    from numpy.random import rand
    return 360.0 * rand(*shape) - 180.0

def gen_random_lat_array(shape):
    from numpy.random import rand
    return 180.0 * rand(*shape) - 90.0

def gen_random_data_array(shape, mean=0.0, var=1.0):
    from numpy.random import randn
    return var*randn(*shape) + mean



'''
Routines for the info command
'''
import sys

def __print_variables(all_variables, user_variables=None, print_err=True):
    '''
        Short routine for printing all variables, or a specified few.

        @param all_variables:   All of the variables to print or search through
        @param user_variables:   The user specified variables of interest
        @param print_err:   Boolean for deciding to print an error if a variable isn't found

    '''
    if user_variables is not None:
        for user_var in user_variables:
            try:
                print user_var+": "+str(all_variables[user_var])
            except KeyError:
                if print_err: sys.stderr.write("Variable '" + user_var +  "' not found \n")
    else:
        for item in all_variables:
            print item


def info(filename, user_variables=None, data_type=None):
    '''
    Read all the variables from a file and print to stdout.
    File can contain either gridded and ungridded data.
    First tries to read data as gridded, if that fails, tries as ungridded.

    @param filenames:   The filenames of the files to read
    @param user_variables:   The user specified variables of interest
    @param data_type:   String representing the type of HDF data to read, i.e. 'VD' or 'SD'

    '''
    from jasmin_cis.data_io.hdf import get_hdf4_file_variables
    from jasmin_cis.data_io.netcdf import get_netcdf_file_variables
    from jasmin_cis.data_io.aeronet import get_aeronet_file_variables
    from jasmin_cis.exceptions import InvalidFileExtensionError, CISError
    from os.path import basename

    if filename.endswith(".nc"):
        try:
            file_variables = get_netcdf_file_variables(filename)
            __print_variables(file_variables, user_variables)
        except RuntimeError as e:
            raise CISError(e)
    elif filename.endswith(".hdf"):
        if (basename(filename).startswith("CAL_LID") or
        basename(filename).startswith("MISR")) and data_type == None:
            data_type = "SD"
        sd_vars, vd_vars = get_hdf4_file_variables(filename, data_type)

        if sd_vars is not None:
            print "\n====== SD variables:"
            __print_variables(sd_vars, user_variables, False)
        if vd_vars is not None:
            print "\n====== VD variables:"
            __print_variables(vd_vars, user_variables, False)
    elif filename.endswith(".lev20"):
        file_variables = get_aeronet_file_variables(filename)
        __print_variables(file_variables, user_variables)
    else:
        raise InvalidFileExtensionError("File extension not recognised. Please use one of .nc, .hdf or .lev20")


from jasmin_cis.plotting.generic_plot import Generic_Plot

class Scatter_Plot(Generic_Plot):

    def plot(self):
        '''
        Plots one or many scatter plots
        Stores the plot in a list to be used for when adding the legend
        '''
        self.plots = []
        scatter_size = self.plot_args.get("itemwidth", 1) if self.plot_args.get("itemwidth", 1) is not None else 1
        for i, unpacked_data_item in enumerate(self.unpacked_data_items):
            datafile = self.plot_args["datagroups"][i]
            if datafile["itemstyle"]:
                self.mplkwargs["marker"] = datafile["itemstyle"]
            else:
                self.mplkwargs.pop("marker", None)

            self.mplkwargs["c"] = datafile.get("color", None)
            if self.mplkwargs["c"] is None:
                if unpacked_data_item.get("y", None) is not None: # i.e. the scatter plot is 3D
                    self.mplkwargs["c"] = unpacked_data_item["data"]
                else:
                    self.mplkwargs.pop("c", None)

            if datafile["edgecolor"]:
                edge_color = datafile["edgecolor"]
            else:
                edge_color = "None"

            x_coords = unpacked_data_item["x"]

            if unpacked_data_item.get("y", None) is not None:
                #3D
                self.scatter_type = "3D"
                y_coords = unpacked_data_item["y"]
            else:
                #2D
                self.scatter_type = "2D"
                y_coords = unpacked_data_item["data"]


            self.plots.append(self.plotting_library.scatter(x_coords, y_coords, s = scatter_size, edgecolors = edge_color, *self.mplargs, **self.mplkwargs))

    def calculate_axis_limits(self, axis, min_val, max_val, step):
        '''
        @param axis: The axis to calculate the limits for
        @param min_val: The user specified minimum value for the axis
        @param max_val: The user specified maximum value for the axis
        @param step: The distance between each tick on the axis
        @return: A dictionary containing the min and max values for the axis, and the step between each tick
        '''
        if axis == "x":
            coord_axis = "x"
        elif axis == "y":
            if self.scatter_type == "2D":
                coord_axis = "data"
            elif self.scatter_type == "3D":
                coord_axis = "y"
        calculated_min, calculated_max = self.calculate_min_and_max_values_of_array_including_case_of_log(axis, self.unpacked_data_items[0][coord_axis])

        valrange = {}
        valrange[axis + "min"] = calculated_min if min_val is None else min_val
        valrange[axis + "max"] = calculated_max if max_val is None else max_val
        valrange[axis + "step"] = step

        return valrange

    def format_plot(self):
        self.format_time_axis()
        if self.scatter_type == "3D":
            self.format_3d_plot()
        elif self.scatter_type == "2D":
            self.format_2d_plot()

    def set_default_axis_label(self, axis):
        import jasmin_cis.exceptions as cisex
        import iris.exceptions as irisex
        axis = axis.lower()
        axislabel = axis + "label"

        if self.plot_args[axislabel] is None:
            if self.is_map():
                self.plot_args[axislabel] = "Longitude" if axis == "x" else "Latitude"
            else:
                try:
                    units = self.packed_data_items[0].coord(name=self.plot_args[axis + "_variable"]).units
                except (cisex.CoordinateNotFoundError, irisex.CoordinateNotFoundError):
                    units = self.packed_data_items[0].units

                if len(self.packed_data_items) == 1:
                    # only 1 data to plot, display
                    try:
                        name = self.packed_data_items[0].coord(name=self.plot_args[axis + "_variable"]).name()
                    except (cisex.CoordinateNotFoundError, irisex.CoordinateNotFoundError):
                        name = self.packed_data_items[0].name()
                    self.plot_args[axislabel] = name + " " + self.format_units(units)
                else:
                    # if more than 1 data, legend will tell us what the name is. so just displaying units
                    self.plot_args[axislabel] = units

    def create_legend(self):
        legend_titles = []
        datagroups = self.plot_args["datagroups"]
        for i, item in enumerate(self.packed_data_items):
            if datagroups is not None and datagroups[i]["label"]:
                legend_titles.append(datagroups[i]["label"])
            else:
                legend_titles.append(item.long_name)
        handles = self.plots
        legend = self.matplotlib.legend(handles, legend_titles, loc="best", scatterpoints = 1, markerscale = 0.5)
        legend.draggable(state = True)


__author__ = 'duncan'
# import re
#
# print re.match(r'.*xenida.*\.nc','/home/duncan/xenida.pah9450.nc')
# print re.match(r'.*2B.CWC.RVOD*','2007180125457_06221_CS_2B-CWC-RVOD_GRANULE_P_R04_E02.hdf')
#
# print re.match(r'.*2B.CWC.RVOD*','/home/duncan/2007180125457_06221_CS_2B-CWC-RVOD_GRANULE_P_R04_E02.hdf')
# print re.match(r'.*\.nc','/home/duncan/2007180125457_06221_CS_2B-CWC-RVOD_GRANULE_P_R04_E02.hdf')
#
# print re.match(r'.*ESACCI.*', "/home/daniel/CCI_Files/20080612093821-ESACCI-L2P_AEROSOL-ALL-AATSR_ENVISAT-ORAC_32855-fv02.02.nc")
#
# import iris
# variable = 'mass_fraction_of_cloud_liquid_water_in_air'
# filename = '/home/duncan/xenida.pah9470.nc'
#
# var_constraint = iris.AttributeConstraint(name=variable)
# # Create an Attribute constraint on the name Attribute for the variable given
#
# cubes = iris.load(filename, variable)
# for cube in cubes:
#     print cube
#
# print cubes.extract(var_constraint)

#cube = iris.load_cube(filename, var_constraint)
#print cube
#
# try:
#     cube = iris.load_cube(filename, var_constraint)
# except iris.exceptions.ConstraintMismatchError:
#     raise InvalidVariableError("Variable not found: " + variable +
#                                "\nTo see a list of variables run: cis info " + filenames[0] + " -h")

import timeit
from jasmin_cis.test.test_col import Test_nn_gridded

tc = Test_nn_gridded()

t = timeit.Timer(tc.test_basic_col_gridded_to_ungridded_in_2d_with_time)

print t.timeit(100)


from plotting.plot import Plotter
from jasmin_cis.test.test_util.mock import make_dummy_1d_ungridded_data
data_object1 = make_dummy_1d_ungridded_data()
data_object2 = make_dummy_1d_ungridded_data()
Plotter([data_object1, data_object2],
        datagroups = [{"itemstyle" : None, "color" : None, "label" : "Line 1"},
                                        {"itemstyle" : None, "color" : None, "label" : None}],
        xrange = {"xmin" : -100, "xstep" : 25, "xmax" : 100},
        yrange = {"ymin" : 0, "ystep" : 0.5},
        plot_type="line")



"""
module to test the time conversion utilities
"""
from nose.tools import istest, raises, eq_, assert_almost_equal
import numpy as np
import datetime as dt

from jasmin_cis.time_util import *

sec = 1.0/(24.0*60.0*60.0)
days_since_standard_epoch = 143541.0 # Almost, but not quite 365.2425*393.0, not sure why...

@istest
def test_that_can_convert_tai_to_datetime_obj():
    a = np.arange(6).reshape(2,3)
    b= convert_sec_since_to_std_time_array(a,dt.datetime(1993,1,1))
    eq_(a.shape,b.shape)
    assert_almost_equal(b[0][0],days_since_standard_epoch)
    assert_almost_equal(b[0][1],days_since_standard_epoch+1*sec)
    assert_almost_equal(b[0][2],days_since_standard_epoch+2*sec)
    assert_almost_equal(b[1][0],days_since_standard_epoch+3*sec)
    assert_almost_equal(b[1][1],days_since_standard_epoch+4*sec)
    assert_almost_equal(b[1][2],days_since_standard_epoch+5*sec)


@istest
def test_that_can_parse_datetimestr_to_obj():
    # when not specifying the hours, minutes or seconds, 0 is used
    eq_(parse_datetimestr_to_std_time("2010-02-05 02:15:45"),convert_datetime_to_std_time(dt.datetime(2010,02,05,02,15,45)))
    eq_(parse_datetimestr_to_std_time("2010-02-05 02:15"),convert_datetime_to_std_time(dt.datetime(2010,02,05,02,15,0)))
    eq_(parse_datetimestr_to_std_time("2010-02-05 02"),convert_datetime_to_std_time(dt.datetime(2010,02,05,02,0,0)))
    eq_(parse_datetimestr_to_std_time("2010-02-05"),convert_datetime_to_std_time(dt.datetime(2010,02,05,0,0,0)))

    # GOTCHA: when not specifying an element of a date (i.e. the year, month or day), the current date is used
    now = dt.datetime.now()
    eq_(parse_datetimestr_to_std_time("2010-02-05"),convert_datetime_to_std_time(dt.datetime(2010,02,05)))
    eq_(parse_datetimestr_to_std_time("2010-02"),convert_datetime_to_std_time(dt.datetime(2010,02,now.day)))
    eq_(parse_datetimestr_to_std_time("2010-"),convert_datetime_to_std_time(dt.datetime(2010,now.month,now.day)))
    eq_(parse_datetimestr_to_std_time(""),convert_datetime_to_std_time(dt.datetime(now.year,now.month,now.day)))


@istest
def test_that_can_parse_time_deltas():
    delta = parse_datetimestr_delta_to_float_days("2y15m3d5M10H3S")
    assert_almost_equal(1190.45829861, delta)

@istest
@raises(ValueError)
def test_that_raise_an_error_when_datetimestr_delta_is_invalid():
    parse_datetimestr_delta_to_float_days("some wierd string")


def test_that_can_calculate_mid_point_between_two_datetime():
    t1 = convert_datetime_to_std_time(dt.datetime(2010,02,05,0,0,0))
    t2 = convert_datetime_to_std_time(dt.datetime(2010,02,06,0,0,0))
    tm = calculate_mid_time(t1,t2)
    eq_(tm,convert_datetime_to_std_time(dt.datetime(2010,02,05,12,0,0)))

